@@ -23,7 +23,9 @@
 #include <cstring>
 #include <sstream>
 #include <fstream>
+#include <string>
 #include <strings.h>
+#include <utility>
 #include <vector>
 
 #include "s3fs.h"
@@ -34,7 +36,7 @@
 //-------------------------------------------------------------------
 // Symbols
 //-------------------------------------------------------------------
-#define ADD_HEAD_REGEX              "reg:"
+static constexpr char ADD_HEAD_REGEX[] = "reg:";
 
 //-------------------------------------------------------------------
 // Class AdditionalHeader
@@ -65,7 +67,7 @@ AdditionalHeader::~AdditionalHeader()
 bool AdditionalHeader::Load(const char* file)
 {
     if(!file){
-        S3FS_PRN_WARN("file is NULL.");
+        S3FS_PRN_WARN("file is nullptr.");
         return false;
     }
     Unload();
@@ -78,7 +80,6 @@ bool AdditionalHeader::Load(const char* file)
 
     // read file
     std::string line;
-    ADDHEAD *paddhead;
     while(getline(AH, line)){
         if(line.empty()){
             continue;
@@ -111,45 +112,30 @@ bool AdditionalHeader::Load(const char* file)
             return false;
         }
 
-        paddhead = new ADDHEAD;
         if(0 == strncasecmp(key.c_str(), ADD_HEAD_REGEX, strlen(ADD_HEAD_REGEX))){
             // regex
             if(key.size() <= strlen(ADD_HEAD_REGEX)){
                 S3FS_PRN_ERR("file format error: %s key(suffix) does not have key std::string.", key.c_str());
-                delete paddhead;
                 continue;
             }
             key.erase(0, strlen(ADD_HEAD_REGEX));
 
-          // compile
-          regex_t*  preg = new regex_t;
-          int       result;
-          if(0 != (result = regcomp(preg, key.c_str(), REG_EXTENDED | REG_NOSUB))){ // we do not need matching info
-              char    errbuf[256];
-              regerror(result, preg, errbuf, sizeof(errbuf));
-              S3FS_PRN_ERR("failed to compile regex from %s key by %s.", key.c_str(), errbuf);
-              delete preg;
-              delete paddhead;
-              continue;
-          }
-
-          // set
-          paddhead->pregex     = preg;
-          paddhead->basestring = key;
-          paddhead->headkey    = head;
-          paddhead->headvalue  = value;
+            // compile
+            RegexPtr preg(new regex_t, regfree);
+            int       result;
+            if(0 != (result = regcomp(preg.get(), key.c_str(), REG_EXTENDED | REG_NOSUB))){ // we do not need matching info
+                char    errbuf[256];
+                regerror(result, preg.get(), errbuf, sizeof(errbuf));
+                S3FS_PRN_ERR("failed to compile regex from %s key by %s.", key.c_str(), errbuf);
+                continue;
+            }
 
+            addheadlist.emplace_back(std::move(preg), key, head, value);
         }else{
             // not regex, directly comparing
-            paddhead->pregex     = NULL;
-            paddhead->basestring = key;
-            paddhead->headkey    = head;
-            paddhead->headvalue  = value;
+            addheadlist.emplace_back(RegexPtr(nullptr, regfree), key, head, value);
         }
 
-        // add list
-        addheadlist.push_back(paddhead);
-
         // set flag
         is_enable = true;
     }
@@ -160,16 +146,6 @@ void AdditionalHeader::Unload()
 {
     is_enable = false;
 
-    for(addheadlist_t::iterator iter = addheadlist.begin(); iter != addheadlist.end(); ++iter){
-        ADDHEAD *paddhead = *iter;
-        if(paddhead){
-            if(paddhead->pregex){
-                regfree(paddhead->pregex);
-                delete paddhead->pregex;
-            }
-            delete paddhead;
-        }
-    }
     addheadlist.clear();
 }
 
@@ -179,7 +155,7 @@ bool AdditionalHeader::AddHeader(headers_t& meta, const char* path) const
         return true;
     }
     if(!path){
-        S3FS_PRN_WARN("path is NULL.");
+        S3FS_PRN_WARN("path is nullptr.");
         return false;
     }
 
@@ -190,23 +166,20 @@ bool AdditionalHeader::AddHeader(headers_t& meta, const char* path) const
     // [NOTE]
     // Because to allow duplicate key, and then scanning the entire table.
     //
-    for(addheadlist_t::const_iterator iter = addheadlist.begin(); iter != addheadlist.end(); ++iter){
-        const ADDHEAD *paddhead = *iter;
-        if(!paddhead){
-            continue;
-        }
+    for(auto iter = addheadlist.cbegin(); iter != addheadlist.cend(); ++iter){
+        const add_header *paddhead = &*iter;
 
         if(paddhead->pregex){
             // regex
             regmatch_t match;         // not use
-            if(0 == regexec(paddhead->pregex, path, 1, &match, 0)){
+            if(0 == regexec(paddhead->pregex.get(), path, 1, &match, 0)){
                 // match -> adding header
                 meta[paddhead->headkey] = paddhead->headvalue;
             }
         }else{
             // directly comparing
             if(paddhead->basestring.length() < pathlength){
-                if(paddhead->basestring.empty() || 0 == strcmp(&path[pathlength - paddhead->basestring.length()], paddhead->basestring.c_str())){
+                if(paddhead->basestring.empty() || paddhead->basestring == &path[pathlength - paddhead->basestring.length()]){
                     // match -> adding header
                     meta[paddhead->headkey] = paddhead->headvalue;
                 }
@@ -223,7 +196,7 @@ struct curl_slist* AdditionalHeader::AddHeader(struct curl_slist* list, const ch
     if(!AddHeader(meta, path)){
         return list;
     }
-    for(headers_t::iterator iter = meta.begin(); iter != meta.end(); ++iter){
+    for(auto iter = meta.cbegin(); iter != meta.cend(); ++iter){
         // Adding header
         list = curl_slist_sort_insert(list, iter->first.c_str(), iter->second.c_str());
     }
@@ -243,20 +216,18 @@ bool AdditionalHeader::Dump() const
 
     ssdbg << "Additional Header list[" << addheadlist.size() << "] = {" << std::endl;
 
-    for(addheadlist_t::const_iterator iter = addheadlist.begin(); iter != addheadlist.end(); ++iter, ++cnt){
-      const ADDHEAD *paddhead = *iter;
+    for(auto iter = addheadlist.cbegin(); iter != addheadlist.cend(); ++iter, ++cnt){
+        const add_header *paddhead = &*iter;
 
-      ssdbg << "    [" << cnt << "] = {" << std::endl;
+        ssdbg << "    [" << cnt << "] = {" << std::endl;
 
-      if(paddhead){
-          if(paddhead->pregex){
-              ssdbg << "        type\t\t--->\tregex" << std::endl;
-          }else{
-              ssdbg << "        type\t\t--->\tsuffix matching" << std::endl;
-          }
-            ssdbg << "        base std::string\t--->\t" << paddhead->basestring << std::endl;
-            ssdbg << "        add header\t--->\t"  << paddhead->headkey << ": " << paddhead->headvalue << std::endl;
+        if(paddhead->pregex){
+            ssdbg << "        type\t\t--->\tregex" << std::endl;
+        }else{
+            ssdbg << "        type\t\t--->\tsuffix matching" << std::endl;
         }
+        ssdbg << "        base std::string\t--->\t" << paddhead->basestring << std::endl;
+        ssdbg << "        add header\t--->\t"  << paddhead->headkey << ": " << paddhead->headvalue << std::endl;
         ssdbg << "    }" << std::endl;
     }
 
