@@ -21,19 +21,21 @@
 #include <algorithm>
 #include <cerrno>
 #include <cstdlib>
+#include <string>
+#include <sys/stat.h>
+#include <utility>
 #include <vector>
 
 #include "s3fs.h"
 #include "s3fs_logger.h"
 #include "s3fs_util.h"
 #include "cache.h"
-#include "autolock.h"
 #include "string_util.h"
 
 //-------------------------------------------------------------------
 // Utility
 //-------------------------------------------------------------------
-inline void SetStatCacheTime(struct timespec& ts)
+static void SetStatCacheTime(struct timespec& ts)
 {
     if(-1 == clock_gettime(static_cast<clockid_t>(CLOCK_MONOTONIC_COARSE), &ts)){
         S3FS_PRN_CRIT("clock_gettime failed: %d", errno);
@@ -41,13 +43,7 @@ inline void SetStatCacheTime(struct timespec& ts)
     }
 }
 
-inline void InitStatCacheTime(struct timespec& ts)
-{
-    ts.tv_sec  = 0;
-    ts.tv_nsec = 0;
-}
-
-inline int CompareStatCacheTime(const struct timespec& ts1, const struct timespec& ts2)
+static int CompareStatCacheTime(const struct timespec& ts1, const struct timespec& ts2)
 {
     // return -1:  ts1 < ts2
     //         0:  ts1 == ts2
@@ -66,7 +62,7 @@ inline int CompareStatCacheTime(const struct timespec& ts1, const struct timespe
     return 0;
 }
 
-inline bool IsExpireStatCacheTime(const struct timespec& ts, const time_t& expire)
+static bool IsExpireStatCacheTime(const struct timespec& ts, time_t expire)
 {
     struct timespec nowts;
     SetStatCacheTime(nowts);
@@ -84,9 +80,9 @@ struct sort_statiterlist{
     // ascending order
     bool operator()(const stat_cache_t::iterator& src1, const stat_cache_t::iterator& src2) const
     {
-        int result = CompareStatCacheTime(src1->second->cache_date, src2->second->cache_date);
+        int result = CompareStatCacheTime(src1->second.cache_date, src2->second.cache_date);
         if(0 == result){
-            if(src1->second->hit_count < src2->second->hit_count){
+            if(src1->second.hit_count < src2->second.hit_count){
                 result = -1;
             }
         }
@@ -103,9 +99,9 @@ struct sort_symlinkiterlist{
     // ascending order
     bool operator()(const symlink_cache_t::iterator& src1, const symlink_cache_t::iterator& src2) const
     {
-        int result = CompareStatCacheTime(src1->second->cache_date, src2->second->cache_date);  // use the same as Stats
+        int result = CompareStatCacheTime(src1->second.cache_date, src2->second.cache_date);  // use the same as Stats
         if(0 == result){
-            if(src1->second->hit_count < src2->second->hit_count){
+            if(src1->second.hit_count < src2->second.hit_count){
                 result = -1;
             }
         }
@@ -117,7 +113,7 @@ struct sort_symlinkiterlist{
 // Static
 //-------------------------------------------------------------------
 StatCache       StatCache::singleton;
-pthread_mutex_t StatCache::stat_cache_lock;
+std::mutex      StatCache::stat_cache_lock;
 
 //-------------------------------------------------------------------
 // Constructor/Destructor
@@ -126,16 +122,6 @@ StatCache::StatCache() : IsExpireTime(true), IsExpireIntervalType(false), Expire
 {
     if(this == StatCache::getStatCacheData()){
         stat_cache.clear();
-        pthread_mutexattr_t attr;
-        pthread_mutexattr_init(&attr);
-#if S3FS_PTHREAD_ERRORCHECK
-        pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
-#endif
-        int result;
-        if(0 != (result = pthread_mutex_init(&StatCache::stat_cache_lock, &attr))){
-            S3FS_PRN_CRIT("failed to init stat_cache_lock: %d", result);
-            abort();
-        }
     }else{
         abort();
     }
@@ -145,11 +131,6 @@ StatCache::~StatCache()
 {
     if(this == StatCache::getStatCacheData()){
         Clear();
-        int result = pthread_mutex_destroy(&StatCache::stat_cache_lock);
-        if(result != 0){
-            S3FS_PRN_CRIT("failed to destroy stat_cache_lock: %d", result);
-            abort();
-        }
     }else{
         abort();
     }
@@ -202,11 +183,8 @@ bool StatCache::SetCacheNoObject(bool flag)
 
 void StatCache::Clear()
 {
-    AutoLock lock(&StatCache::stat_cache_lock);
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
 
-    for(stat_cache_t::iterator iter = stat_cache.begin(); iter != stat_cache.end(); ++iter){
-        delete (*iter).second;
-    }
     stat_cache.clear();
     S3FS_MALLOCTRIM(0);
 }
@@ -216,9 +194,9 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
     bool is_delete_cache = false;
     std::string strpath = key;
 
-    AutoLock lock(&StatCache::stat_cache_lock);
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
 
-    stat_cache_t::iterator iter = stat_cache.end();
+    auto iter = stat_cache.end();
     if(overcheck && '/' != *strpath.rbegin()){
         strpath += "/";
         iter = stat_cache.find(strpath);
@@ -228,13 +206,13 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
         iter = stat_cache.find(strpath);
     }
 
-    if(iter != stat_cache.end() && (*iter).second){
-        stat_cache_entry* ent = (*iter).second;
+    if(iter != stat_cache.end()){
+        stat_cache_entry* ent = &iter->second;
         if(0 < ent->notruncate || !IsExpireTime || !IsExpireStatCacheTime(ent->cache_date, ExpireTime)){
             if(ent->noobjcache){
                 if(!IsCacheNoObject){
                     // need to delete this cache.
-                    DelStat(strpath, AutoLock::ALREADY_LOCKED);
+                    DelStatHasLock(strpath);
                 }else{
                     // noobjcache = true means no object.
                 }
@@ -244,11 +222,11 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
             std::string stretag;
             if(petag){
                 // find & check ETag
-                for(headers_t::iterator hiter = ent->meta.begin(); hiter != ent->meta.end(); ++hiter){
+                for(auto hiter = ent->meta.cbegin(); hiter != ent->meta.cend(); ++hiter){
                     std::string tag = lower(hiter->first);
                     if(tag == "etag"){
                         stretag = hiter->second;
-                        if('\0' != petag[0] && 0 != strcmp(petag, stretag.c_str())){
+                        if('\0' != petag[0] && petag != stretag){
                             is_delete_cache = true;
                         }
                         break;
@@ -264,13 +242,13 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
                 S3FS_PRN_DBG("stat cache hit [path=%s][time=%lld.%09ld][hit count=%lu]",
                     strpath.c_str(), static_cast<long long>(ent->cache_date.tv_sec), ent->cache_date.tv_nsec, ent->hit_count);
 
-                if(pst!= NULL){
+                if(pst!= nullptr){
                     *pst= ent->stbuf;
                 }
-                if(meta != NULL){
+                if(meta != nullptr){
                     *meta = ent->meta;
                 }
-                if(pisforce != NULL){
+                if(pisforce != nullptr){
                     (*pisforce) = ent->isforce;
                 }
                 ent->hit_count++;
@@ -288,7 +266,7 @@ bool StatCache::GetStat(const std::string& key, struct stat* pst, headers_t* met
     }
 
     if(is_delete_cache){
-        DelStat(strpath, AutoLock::ALREADY_LOCKED);
+        DelStatHasLock(strpath);
     }
     return false;
 }
@@ -302,9 +280,9 @@ bool StatCache::IsNoObjectCache(const std::string& key, bool overcheck)
         return false;
     }
 
-    AutoLock lock(&StatCache::stat_cache_lock);
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
 
-    stat_cache_t::iterator iter = stat_cache.end();
+    auto iter = stat_cache.end();
     if(overcheck && '/' != *strpath.rbegin()){
         strpath += "/";
         iter     = stat_cache.find(strpath);
@@ -314,12 +292,12 @@ bool StatCache::IsNoObjectCache(const std::string& key, bool overcheck)
         iter    = stat_cache.find(strpath);
     }
 
-    if(iter != stat_cache.end() && (*iter).second) {
-        stat_cache_entry* ent = (*iter).second;
-        if(0 < ent->notruncate || !IsExpireTime || !IsExpireStatCacheTime((*iter).second->cache_date, ExpireTime)){
-            if((*iter).second->noobjcache){
+    if(iter != stat_cache.end()) {
+        const stat_cache_entry* ent = &iter->second;
+        if(0 < ent->notruncate || !IsExpireTime || !IsExpireStatCacheTime(iter->second.cache_date, ExpireTime)){
+            if(iter->second.noobjcache){
                 // noobjcache = true means no object.
-                SetStatCacheTime((*iter).second->cache_date);
+                SetStatCacheTime((*iter).second.cache_date);
                 return true;
             }
         }else{
@@ -329,7 +307,7 @@ bool StatCache::IsNoObjectCache(const std::string& key, bool overcheck)
     }
 
     if(is_delete_cache){
-        DelStat(strpath, AutoLock::ALREADY_LOCKED);
+        DelStatHasLock(strpath);
     }
     return false;
 }
@@ -341,18 +319,16 @@ bool StatCache::AddStat(const std::string& key, const headers_t& meta, bool forc
     }
     S3FS_PRN_INFO3("add stat cache entry[path=%s]", key.c_str());
 
-    bool found;
-    bool do_truncate;
-    {
-        AutoLock lock(&StatCache::stat_cache_lock);
-        found       = stat_cache.end() != stat_cache.find(key);
-        do_truncate = stat_cache.size() > CacheSize;
-    }
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
 
-    if(found){
-        DelStat(key.c_str());
+    if(stat_cache.cend() != stat_cache.find(key)){
+        // found cache
+        DelStatHasLock(key);
     }else{
-        if(do_truncate){
+        // check: need to truncate cache
+        if(stat_cache.size() > CacheSize){
+            // cppcheck-suppress unmatchedSuppression
+            // cppcheck-suppress knownConditionTrueFalse
             if(!TruncateCache()){
                 return false;
             }
@@ -360,50 +336,49 @@ bool StatCache::AddStat(const std::string& key, const headers_t& meta, bool forc
     }
 
     // make new
-    stat_cache_entry* ent = new stat_cache_entry();
-    if(!convert_header_to_stat(key.c_str(), meta, &(ent->stbuf), forcedir)){
-        delete ent;
+    stat_cache_entry ent;
+    if(!convert_header_to_stat(key.c_str(), meta, &ent.stbuf, forcedir)){
         return false;
     }
-    ent->hit_count  = 0;
-    ent->isforce    = forcedir;
-    ent->noobjcache = false;
-    ent->notruncate = (no_truncate ? 1L : 0L);
-    ent->meta.clear();
-    SetStatCacheTime(ent->cache_date);    // Set time.
+    ent.hit_count  = 0;
+    ent.isforce    = forcedir;
+    ent.noobjcache = false;
+    ent.notruncate = (no_truncate ? 1L : 0L);
+    ent.meta.clear();
+    SetStatCacheTime(ent.cache_date);    // Set time.
     //copy only some keys
-    for(headers_t::const_iterator iter = meta.begin(); iter != meta.end(); ++iter){
+    for(auto iter = meta.cbegin(); iter != meta.cend(); ++iter){
         std::string tag   = lower(iter->first);
         std::string value = iter->second;
         if(tag == "content-type"){
-            ent->meta[iter->first] = value;
+            ent.meta[iter->first] = value;
         }else if(tag == "content-length"){
-            ent->meta[iter->first] = value;
+            ent.meta[iter->first] = value;
         }else if(tag == "etag"){
-            ent->meta[iter->first] = value;
+            ent.meta[iter->first] = value;
         }else if(tag == "last-modified"){
-            ent->meta[iter->first] = value;
+            ent.meta[iter->first] = value;
         }else if(is_prefix(tag.c_str(), "x-amz")){
-            ent->meta[tag] = value;      // key is lower case for "x-amz"
+            ent.meta[tag] = value;      // key is lower case for "x-amz"
         }
     }
 
-    // add
-    AutoLock lock(&StatCache::stat_cache_lock);
-
-    std::pair<stat_cache_t::iterator, bool> pair = stat_cache.insert(std::make_pair(key, ent));
-    if(!pair.second){
-        delete pair.first->second;
-        pair.first->second = ent;
-    }
+    const auto& value = stat_cache[key] = std::move(ent);
 
     // check symbolic link cache
-    if(!S_ISLNK(ent->stbuf.st_mode)){
-        if(symlink_cache.end() != symlink_cache.find(key)){
+    if(!S_ISLNK(value.stbuf.st_mode)){
+        if(symlink_cache.cend() != symlink_cache.find(key)){
             // if symbolic link cache has key, thus remove it.
-            DelSymlink(key.c_str(), AutoLock::ALREADY_LOCKED);
+            DelSymlinkHasLock(key);
         }
     }
+
+    // If no_truncate flag is set, set file name to notruncate_file_cache
+    //
+    if(no_truncate){
+        AddNotruncateCache(key);
+    }
+
     return true;
 }
 
@@ -414,22 +389,22 @@ bool StatCache::AddStat(const std::string& key, const headers_t& meta, bool forc
 // Since the file mode may change while the file is open, it is
 // updated as well.
 //
-bool StatCache::UpdateMetaStats(const std::string& key, headers_t& meta)
+bool StatCache::UpdateMetaStats(const std::string& key, const headers_t& meta)
 {
     if(CacheSize < 1){
         return true;
     }
     S3FS_PRN_INFO3("update stat cache entry[path=%s]", key.c_str());
 
-    AutoLock lock(&StatCache::stat_cache_lock);
-    stat_cache_t::iterator iter = stat_cache.find(key);
-    if(stat_cache.end() == iter || !(iter->second)){
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
+    auto iter = stat_cache.find(key);
+    if(stat_cache.cend() == iter){
         return true;
     }
-    stat_cache_entry* ent = iter->second;
+    stat_cache_entry* ent = &iter->second;
 
     // update only meta keys
-    for(headers_t::iterator metaiter = meta.begin(); metaiter != meta.end(); ++metaiter){
+    for(auto metaiter = meta.cbegin(); metaiter != meta.cend(); ++metaiter){
         std::string tag   = lower(metaiter->first);
         std::string value = metaiter->second;
         if(tag == "content-type"){
@@ -464,18 +439,16 @@ bool StatCache::AddNoObjectCache(const std::string& key)
     }
     S3FS_PRN_INFO3("add no object cache entry[path=%s]", key.c_str());
 
-    bool found;
-    bool do_truncate;
-    {
-        AutoLock lock(&StatCache::stat_cache_lock);
-        found       = stat_cache.end() != stat_cache.find(key);
-        do_truncate = stat_cache.size() > CacheSize;
-    }
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
 
-    if(found){
-        DelStat(key.c_str());
+    if(stat_cache.cend() != stat_cache.find(key)){
+		// found
+        DelStatHasLock(key);
     }else{
-        if(do_truncate){
+        // check: need to truncate cache
+        if(stat_cache.size() > CacheSize){
+            // cppcheck-suppress unmatchedSuppression
+            // cppcheck-suppress knownConditionTrueFalse
             if(!TruncateCache()){
                 return false;
             }
@@ -483,45 +456,43 @@ bool StatCache::AddNoObjectCache(const std::string& key)
     }
 
     // make new
-    stat_cache_entry* ent = new stat_cache_entry();
-    memset(&(ent->stbuf), 0, sizeof(struct stat));
-    ent->hit_count  = 0;
-    ent->isforce    = false;
-    ent->noobjcache = true;
-    ent->notruncate = 0L;
-    ent->meta.clear();
-    SetStatCacheTime(ent->cache_date);    // Set time.
+    stat_cache_entry ent{};
+    ent.hit_count  = 0;
+    ent.isforce    = false;
+    ent.noobjcache = true;
+    ent.notruncate = 0L;
+    ent.meta.clear();
+    SetStatCacheTime(ent.cache_date);    // Set time.
 
-    // add
-    AutoLock lock(&StatCache::stat_cache_lock);
-
-    std::pair<stat_cache_t::iterator, bool> pair = stat_cache.insert(std::make_pair(key, ent));
-    if(!pair.second){
-        delete pair.first->second;
-        pair.first->second = ent;
-    }
+    stat_cache[key] = std::move(ent);
 
     // check symbolic link cache
-    if(symlink_cache.end() != symlink_cache.find(key)){
+    if(symlink_cache.cend() != symlink_cache.find(key)){
         // if symbolic link cache has key, thus remove it.
-        DelSymlink(key.c_str(), AutoLock::ALREADY_LOCKED);
+        DelSymlinkHasLock(key);
     }
     return true;
 }
 
 void StatCache::ChangeNoTruncateFlag(const std::string& key, bool no_truncate)
 {
-    AutoLock lock(&StatCache::stat_cache_lock);
-    stat_cache_t::iterator iter = stat_cache.find(key);
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
+    auto iter = stat_cache.find(key);
 
-    if(stat_cache.end() != iter){
-        stat_cache_entry* ent = iter->second;
-        if(ent){
-            if(no_truncate){
-                ++(ent->notruncate);
-            }else{
-                if(0L < ent->notruncate){
-                    --(ent->notruncate);
+    if(stat_cache.cend() != iter){
+        stat_cache_entry* ent = &iter->second;
+        if(no_truncate){
+            if(0L == ent->notruncate){
+                // need to add no truncate cache.
+                AddNotruncateCache(key);
+            }
+            ++(ent->notruncate);
+        }else{
+            if(0L < ent->notruncate){
+                --(ent->notruncate);
+                if(0L == ent->notruncate){
+                    // need to delete from no truncate cache.
+                    DelNotruncateCache(key);
                 }
             }
         }
@@ -530,19 +501,16 @@ void StatCache::ChangeNoTruncateFlag(const std::string& key, bool no_truncate)
 
 bool StatCache::TruncateCache()
 {
-    AutoLock lock(&StatCache::stat_cache_lock);
-
     if(stat_cache.empty()){
         return true;
     }
 
     // 1) erase over expire time
     if(IsExpireTime){
-        for(stat_cache_t::iterator iter = stat_cache.begin(); iter != stat_cache.end(); ){
-            stat_cache_entry* entry = iter->second;
-            if(!entry || (0L == entry->notruncate && IsExpireStatCacheTime(entry->cache_date, ExpireTime))){
-                delete entry;
-                stat_cache.erase(iter++);
+        for(auto iter = stat_cache.cbegin(); iter != stat_cache.cend(); ){
+            const stat_cache_entry* entry = &iter->second;
+            if(0L == entry->notruncate && IsExpireStatCacheTime(entry->cache_date, ExpireTime)){
+                iter = stat_cache.erase(iter);
             }else{
                 ++iter;
             }
@@ -557,10 +525,10 @@ bool StatCache::TruncateCache()
     // 3) erase from the old cache in order
     size_t            erase_count= stat_cache.size() - CacheSize + 1;
     statiterlist_t    erase_iters;
-    for(stat_cache_t::iterator iter = stat_cache.begin(); iter != stat_cache.end() && 0 < erase_count; ++iter){
+    for(auto iter = stat_cache.begin(); iter != stat_cache.end() && 0 < erase_count; ++iter){
         // check no truncate
-        stat_cache_entry* ent = iter->second;
-        if(ent && 0L < ent->notruncate){
+        const stat_cache_entry* ent = &iter->second;
+        if(0L < ent->notruncate){
             // skip for no truncate entry and keep extra counts for this entity.
             if(0 < erase_count){
                 --erase_count;     // decrement
@@ -576,11 +544,10 @@ bool StatCache::TruncateCache()
             }
         }
     }
-    for(statiterlist_t::iterator iiter = erase_iters.begin(); iiter != erase_iters.end(); ++iiter){
-        stat_cache_t::iterator siter = *iiter;
+    for(auto iiter = erase_iters.cbegin(); iiter != erase_iters.cend(); ++iiter){
+        auto siter = *iiter;
 
         S3FS_PRN_DBG("truncate stat cache[path=%s]", siter->first.c_str());
-        delete siter->second;
         stat_cache.erase(siter);
     }
     S3FS_MALLOCTRIM(0);
@@ -588,21 +555,16 @@ bool StatCache::TruncateCache()
     return true;
 }
 
-bool StatCache::DelStat(const char* key, AutoLock::Type locktype)
+bool StatCache::DelStatHasLock(const std::string& key)
 {
-    if(!key){
-        return false;
-    }
-    S3FS_PRN_INFO3("delete stat cache entry[path=%s]", key);
-
-    AutoLock lock(&StatCache::stat_cache_lock, locktype);
+    S3FS_PRN_INFO3("delete stat cache entry[path=%s]", key.c_str());
 
     stat_cache_t::iterator iter;
-    if(stat_cache.end() != (iter = stat_cache.find(std::string(key)))){
-        delete (*iter).second;
+    if(stat_cache.cend() != (iter = stat_cache.find(key))){
         stat_cache.erase(iter);
+        DelNotruncateCache(key);
     }
-    if(0 < strlen(key) && 0 != strcmp(key, "/")){
+    if(!key.empty() && key != "/"){
         std::string strpath = key;
         if('/' == *strpath.rbegin()){
             // If there is "path" cache, delete it.
@@ -611,9 +573,9 @@ bool StatCache::DelStat(const char* key, AutoLock::Type locktype)
             // If there is "path/" cache, delete it.
             strpath += "/";
         }
-        if(stat_cache.end() != (iter = stat_cache.find(strpath))){
-            delete (*iter).second;
+        if(stat_cache.cend() != (iter = stat_cache.find(strpath))){
             stat_cache.erase(iter);
+            DelNotruncateCache(strpath);
         }
     }
     S3FS_MALLOCTRIM(0);
@@ -626,11 +588,11 @@ bool StatCache::GetSymlink(const std::string& key, std::string& value)
     bool is_delete_cache = false;
     const std::string& strpath = key;
 
-    AutoLock lock(&StatCache::stat_cache_lock);
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
 
-    symlink_cache_t::iterator iter = symlink_cache.find(strpath);
-    if(iter != symlink_cache.end() && iter->second){
-        symlink_cache_entry* ent = iter->second;
+    auto iter = symlink_cache.find(strpath);
+    if(iter != symlink_cache.cend()){
+        symlink_cache_entry* ent = &iter->second;
         if(!IsExpireTime || !IsExpireStatCacheTime(ent->cache_date, ExpireTime)){   // use the same as Stats
             // found
             S3FS_PRN_DBG("symbolic link cache hit [path=%s][time=%lld.%09ld][hit count=%lu]",
@@ -650,7 +612,7 @@ bool StatCache::GetSymlink(const std::string& key, std::string& value)
     }
 
     if(is_delete_cache){
-        DelSymlink(strpath.c_str(), AutoLock::ALREADY_LOCKED);
+        DelSymlinkHasLock(strpath);
     }
     return false;
 }
@@ -662,18 +624,16 @@ bool StatCache::AddSymlink(const std::string& key, const std::string& value)
     }
     S3FS_PRN_INFO3("add symbolic link cache entry[path=%s, value=%s]", key.c_str(), value.c_str());
 
-    bool found;
-    bool do_truncate;
-    {
-        AutoLock lock(&StatCache::stat_cache_lock);
-        found       = symlink_cache.end() != symlink_cache.find(key);
-        do_truncate = symlink_cache.size() > CacheSize;
-    }
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
 
-    if(found){
-        DelSymlink(key.c_str());
+    if(symlink_cache.cend() != symlink_cache.find(key)){
+    	// found
+        DelSymlinkHasLock(key);
     }else{
-        if(do_truncate){
+        // check: need to truncate cache
+        if(symlink_cache.size() > CacheSize){
+            // cppcheck-suppress unmatchedSuppression
+            // cppcheck-suppress knownConditionTrueFalse
             if(!TruncateSymlink()){
                 return false;
             }
@@ -681,38 +641,28 @@ bool StatCache::AddSymlink(const std::string& key, const std::string& value)
     }
 
     // make new
-    symlink_cache_entry* ent = new symlink_cache_entry();
-    ent->link       = value;
-    ent->hit_count  = 0;
-    SetStatCacheTime(ent->cache_date);    // Set time(use the same as Stats).
+    symlink_cache_entry ent;
+    ent.link       = value;
+    ent.hit_count  = 0;
+    SetStatCacheTime(ent.cache_date);    // Set time(use the same as Stats).
 
-    // add
-    AutoLock lock(&StatCache::stat_cache_lock);
-
-    std::pair<symlink_cache_t::iterator, bool> pair = symlink_cache.insert(std::make_pair(key, ent));
-    if(!pair.second){
-        delete pair.first->second;
-        pair.first->second = ent;
-    }
+    symlink_cache[key] = std::move(ent);
 
     return true;
 }
 
 bool StatCache::TruncateSymlink()
 {
-    AutoLock lock(&StatCache::stat_cache_lock);
-
     if(symlink_cache.empty()){
         return true;
     }
 
     // 1) erase over expire time
     if(IsExpireTime){
-        for(symlink_cache_t::iterator iter = symlink_cache.begin(); iter != symlink_cache.end(); ){
-            symlink_cache_entry* entry = iter->second;
-            if(!entry || IsExpireStatCacheTime(entry->cache_date, ExpireTime)){  // use the same as Stats
-                delete entry;
-                symlink_cache.erase(iter++);
+        for(auto iter = symlink_cache.cbegin(); iter != symlink_cache.cend(); ){
+            const symlink_cache_entry* entry = &iter->second;
+            if(IsExpireStatCacheTime(entry->cache_date, ExpireTime)){  // use the same as Stats
+                iter = symlink_cache.erase(iter);
             }else{
                 ++iter;
             }
@@ -727,18 +677,17 @@ bool StatCache::TruncateSymlink()
     // 3) erase from the old cache in order
     size_t            erase_count= symlink_cache.size() - CacheSize + 1;
     symlinkiterlist_t erase_iters;
-    for(symlink_cache_t::iterator iter = symlink_cache.begin(); iter != symlink_cache.end(); ++iter){
+    for(auto iter = symlink_cache.begin(); iter != symlink_cache.end(); ++iter){
         erase_iters.push_back(iter);
         sort(erase_iters.begin(), erase_iters.end(), sort_symlinkiterlist());
         if(erase_count < erase_iters.size()){
             erase_iters.pop_back();
         }
     }
-    for(symlinkiterlist_t::iterator iiter = erase_iters.begin(); iiter != erase_iters.end(); ++iiter){
-        symlink_cache_t::iterator siter = *iiter;
+    for(auto iiter = erase_iters.cbegin(); iiter != erase_iters.cend(); ++iiter){
+        auto siter = *iiter;
 
         S3FS_PRN_DBG("truncate symbolic link  cache[path=%s]", siter->first.c_str());
-        delete siter->second;
         symlink_cache.erase(siter);
     }
     S3FS_MALLOCTRIM(0);
@@ -746,18 +695,12 @@ bool StatCache::TruncateSymlink()
     return true;
 }
 
-bool StatCache::DelSymlink(const char* key, AutoLock::Type locktype)
+bool StatCache::DelSymlinkHasLock(const std::string& key)
 {
-    if(!key){
-        return false;
-    }
-    S3FS_PRN_INFO3("delete symbolic link cache entry[path=%s]", key);
-
-    AutoLock lock(&StatCache::stat_cache_lock, locktype);
+    S3FS_PRN_INFO3("delete symbolic link cache entry[path=%s]", key.c_str());
 
     symlink_cache_t::iterator iter;
-    if(symlink_cache.end() != (iter = symlink_cache.find(std::string(key)))){
-        delete iter->second;
+    if(symlink_cache.cend() != (iter = symlink_cache.find(key))){
         symlink_cache.erase(iter);
     }
     S3FS_MALLOCTRIM(0);
@@ -765,6 +708,110 @@ bool StatCache::DelSymlink(const char* key, AutoLock::Type locktype)
     return true;
 }
 
+bool StatCache::AddNotruncateCache(const std::string& key)
+{
+    if(key.empty() || '/' == *key.rbegin()){
+        return false;
+    }
+
+    std::string parentdir = mydirname(key);
+    std::string filename  = mybasename(key);
+    if(parentdir.empty() || filename.empty()){
+        return false;
+    }
+    parentdir += '/';       // directory path must be '/' termination.
+
+    auto iter = notruncate_file_cache.find(parentdir);
+    if(iter == notruncate_file_cache.cend()){
+        // add new list
+        notruncate_filelist_t list;
+        list.push_back(filename);
+        notruncate_file_cache[parentdir] = list;
+    }else{
+        // add filename to existed list
+        notruncate_filelist_t& filelist = iter->second;
+        auto fiter = std::find(filelist.cbegin(), filelist.cend(), filename);
+        if(fiter == filelist.cend()){
+            filelist.push_back(filename);
+        }
+    }
+    return true;
+}
+
+bool StatCache::DelNotruncateCache(const std::string& key)
+{
+    if(key.empty() || '/' == *key.rbegin()){
+        return false;
+    }
+
+    std::string parentdir = mydirname(key);
+    std::string filename  = mybasename(key);
+    if(parentdir.empty() || filename.empty()){
+        return false;
+    }
+    parentdir += '/';       // directory path must be '/' termination.
+
+    auto iter = notruncate_file_cache.find(parentdir);
+    if(iter != notruncate_file_cache.cend()){
+        // found directory in map
+        notruncate_filelist_t& filelist = iter->second;
+        auto fiter = std::find(filelist.begin(), filelist.end(), filename);
+        if(fiter != filelist.cend()){
+            // found filename in directory file list
+            filelist.erase(fiter);
+            if(filelist.empty()){
+                notruncate_file_cache.erase(parentdir);
+            }
+        }
+    }
+    return true;
+}
+
+// [Background]
+// When s3fs creates a new file, the file does not exist until the file contents
+// are uploaded.(because it doesn't create a 0 byte file)
+// From the time this file is created(opened) until it is uploaded(flush), it
+// will have a Stat cache with the No truncate flag added.
+// This avoids file not existing errors in operations such as chmod and utimens
+// that occur in the short period before file upload.
+// Besides this, we also need to support readdir(list_bucket), this method is
+// called to maintain the cache for readdir and return its value.
+//
+// [NOTE]
+// Add the file names under parentdir to the list.
+// However, if the same file name exists in the list, it will not be added.
+// parentdir must be terminated with a '/'.
+//
+bool StatCache::GetNotruncateCache(const std::string& parentdir, notruncate_filelist_t& list)
+{
+    if(parentdir.empty()){
+        return false;
+    }
+
+    std::string dirpath = parentdir;
+    if('/' != *dirpath.rbegin()){
+        dirpath += '/';
+    }
+
+    const std::lock_guard<std::mutex> lock(StatCache::stat_cache_lock);
+
+    auto iter = notruncate_file_cache.find(dirpath);
+    if(iter == notruncate_file_cache.cend()){
+        // not found directory map
+        return true;
+    }
+
+    // found directory in map
+    const notruncate_filelist_t& filelist = iter->second;
+    for(auto fiter = filelist.cbegin(); fiter != filelist.cend(); ++fiter){
+        if(list.cend() == std::find(list.cbegin(), list.cend(), *fiter)){
+           // found notuncate file that does not exist in the list, so add it.
+           list.push_back(*fiter);
+        }
+    }
+    return true;
+}
+
 //-------------------------------------------------------------------
 // Functions
 //-------------------------------------------------------------------
@@ -773,7 +820,7 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
     if(!path || !pst){
         return false;
     }
-    memset(pst, 0, sizeof(struct stat));
+    *pst = {};
 
     pst->st_nlink = 1; // see fuse FAQ
 
@@ -795,7 +842,7 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
             mtime.tv_sec  = 0;
             mtime.tv_nsec = 0;
         }
-        set_timespec_to_stat(*pst, ST_TYPE_MTIME, mtime);
+        set_timespec_to_stat(*pst, stat_time_type::MTIME, mtime);
     }
 
     // ctime
@@ -807,7 +854,7 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
             ctime.tv_sec  = 0;
             ctime.tv_nsec = 0;
         }
-        set_timespec_to_stat(*pst, ST_TYPE_CTIME, ctime);
+        set_timespec_to_stat(*pst, stat_time_type::CTIME, ctime);
     }
 
     // atime
@@ -819,11 +866,15 @@ bool convert_header_to_stat(const char* path, const headers_t& meta, struct stat
             atime.tv_sec  = 0;
             atime.tv_nsec = 0;
         }
-        set_timespec_to_stat(*pst, ST_TYPE_ATIME, atime);
+        set_timespec_to_stat(*pst, stat_time_type::ATIME, atime);
     }
 
     // size
-    pst->st_size = get_size(meta);
+    if(S_ISDIR(pst->st_mode)){
+        pst->st_size = 4096;
+    }else{
+        pst->st_size = get_size(meta);
+    }
 
     // uid/gid
     pst->st_uid = get_uid(meta);
