@@ -19,66 +19,47 @@
  */
 
 #include <cstdio>
+#include <utility>
 
 #include "s3fs_logger.h"
 #include "curl_handlerpool.h"
-#include "autolock.h"
 
 //-------------------------------------------------------------------
 // Class CurlHandlerPool
 //-------------------------------------------------------------------
 bool CurlHandlerPool::Init()
 {
-    pthread_mutexattr_t attr;
-    pthread_mutexattr_init(&attr);
-#if S3FS_PTHREAD_ERRORCHECK
-    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
-#endif
-    if (0 != pthread_mutex_init(&mLock, &attr)) {
-        S3FS_PRN_ERR("Init curl handlers lock failed");
-        return false;
-    }
-
     for(int cnt = 0; cnt < mMaxHandlers; ++cnt){
-        CURL* hCurl = curl_easy_init();
+        CurlUniquePtr hCurl(curl_easy_init(), &curl_easy_cleanup);
         if(!hCurl){
             S3FS_PRN_ERR("Init curl handlers pool failed");
             Destroy();
             return false;
         }
-        mPool.push_back(hCurl);
+        const std::lock_guard<std::mutex> lock(mLock);
+        mPool.push_back(std::move(hCurl));
     }
     return true;
 }
 
 bool CurlHandlerPool::Destroy()
 {
-    {
-        AutoLock lock(&mLock);
+    const std::lock_guard<std::mutex> lock(mLock);
 
-        while(!mPool.empty()){
-            CURL* hCurl = mPool.back();
-            mPool.pop_back();
-            if(hCurl){
-                curl_easy_cleanup(hCurl);
-            }
-        }
-    }
-    if (0 != pthread_mutex_destroy(&mLock)) {
-        S3FS_PRN_ERR("Destroy curl handlers lock failed");
-        return false;
+    while(!mPool.empty()){
+        mPool.pop_back();
     }
     return true;
 }
 
-CURL* CurlHandlerPool::GetHandler(bool only_pool)
+CurlUniquePtr CurlHandlerPool::GetHandler(bool only_pool)
 {
-    AutoLock lock(&mLock);
+    const std::lock_guard<std::mutex> lock(mLock);
 
-    CURL* hCurl = NULL;
+    CurlUniquePtr hCurl(nullptr, curl_easy_cleanup);
 
     if(!mPool.empty()){
-        hCurl = mPool.back();
+        hCurl = std::move(mPool.back());
         mPool.pop_back();
         S3FS_PRN_DBG("Get handler from pool: rest = %d", static_cast<int>(mPool.size()));
     }
@@ -87,33 +68,28 @@ CURL* CurlHandlerPool::GetHandler(bool only_pool)
     }
     if(!hCurl){
         S3FS_PRN_INFO("Pool empty: force to create new handler");
-        hCurl = curl_easy_init();
+        hCurl.reset(curl_easy_init());
     }
     return hCurl;
 }
 
-void CurlHandlerPool::ReturnHandler(CURL* hCurl, bool restore_pool)
+void CurlHandlerPool::ReturnHandler(CurlUniquePtr&& hCurl, bool restore_pool)
 {
     if(!hCurl){
       return;
     }
-    AutoLock lock(&mLock);
+    const std::lock_guard<std::mutex> lock(mLock);
 
     if(restore_pool){
         S3FS_PRN_DBG("Return handler to pool");
-        mPool.push_back(hCurl);
+        mPool.push_back(std::move(hCurl));
 
         while(mMaxHandlers < static_cast<int>(mPool.size())){
-            CURL* hOldCurl = mPool.front();
+            S3FS_PRN_INFO("Pool full: destroy the oldest handler");
             mPool.pop_front();
-            if(hOldCurl){
-                S3FS_PRN_INFO("Pool full: destroy the oldest handler");
-                curl_easy_cleanup(hOldCurl);
-            }
         }
     }else{
         S3FS_PRN_INFO("Pool full: destroy the handler");
-        curl_easy_cleanup(hCurl);
     }
 }
 
@@ -122,7 +98,7 @@ void CurlHandlerPool::ResetHandler(CURL* hCurl)
     if(!hCurl){
       return;
     }
-    AutoLock lock(&mLock);
+    const std::lock_guard<std::mutex> lock(mLock);
 
     curl_easy_reset(hCurl);
 }
