@@ -21,58 +21,44 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cerrno>
+#include <future>
+#include <thread>
+#include <utility>
 #include <vector>
 
 #include "s3fs.h"
 #include "s3fs_logger.h"
 #include "curl_multi.h"
 #include "curl.h"
-#include "autolock.h"
 #include "psemaphore.h"
 
 //-------------------------------------------------------------------
 // Class S3fsMultiCurl 
 //-------------------------------------------------------------------
-S3fsMultiCurl::S3fsMultiCurl(int maxParallelism) : maxParallelism(maxParallelism), SuccessCallback(NULL), NotFoundCallback(NULL), RetryCallback(NULL), pSuccessCallbackParam(NULL), pNotFoundCallbackParam(NULL)
+S3fsMultiCurl::S3fsMultiCurl(int maxParallelism, bool not_abort) : maxParallelism(maxParallelism), not_abort(not_abort), SuccessCallback(nullptr), NotFoundCallback(nullptr), RetryCallback(nullptr), pSuccessCallbackParam(nullptr), pNotFoundCallbackParam(nullptr)
 {
-    int result;
-    pthread_mutexattr_t attr;
-    pthread_mutexattr_init(&attr);
-#if S3FS_PTHREAD_ERRORCHECK
-    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
-#endif
-    if (0 != (result = pthread_mutex_init(&completed_tids_lock, &attr))) {
-        S3FS_PRN_ERR("could not initialize completed_tids_lock: %i", result);
-        abort();
-    }
 }
 
 S3fsMultiCurl::~S3fsMultiCurl()
 {
     Clear();
-    int result;
-    if(0 != (result = pthread_mutex_destroy(&completed_tids_lock))){
-        S3FS_PRN_ERR("could not destroy completed_tids_lock: %i", result);
-    }
 }
 
 bool S3fsMultiCurl::ClearEx(bool is_all)
 {
-    s3fscurllist_t::iterator iter;
-    for(iter = clist_req.begin(); iter != clist_req.end(); ++iter){
-        S3fsCurl* s3fscurl = *iter;
+    s3fscurllist_t::const_iterator iter;
+    for(iter = clist_req.cbegin(); iter != clist_req.cend(); ++iter){
+        S3fsCurl* s3fscurl = iter->get();
         if(s3fscurl){
             s3fscurl->DestroyCurlHandle();
-            delete s3fscurl;  // with destroy curl handle.
         }
     }
     clist_req.clear();
 
     if(is_all){
-        for(iter = clist_all.begin(); iter != clist_all.end(); ++iter){
-            S3fsCurl* s3fscurl = *iter;
+        for(iter = clist_all.cbegin(); iter != clist_all.cend(); ++iter){
+            S3fsCurl* s3fscurl = iter->get();
             s3fscurl->DestroyCurlHandle();
-            delete s3fscurl;
         }
         clist_all.clear();
     }
@@ -117,27 +103,25 @@ void* S3fsMultiCurl::SetNotFoundCallbackParam(void* param)
     return old;
 }
 
-bool S3fsMultiCurl::SetS3fsCurlObject(S3fsCurl* s3fscurl)
+bool S3fsMultiCurl::SetS3fsCurlObject(std::unique_ptr<S3fsCurl> s3fscurl)
 {
     if(!s3fscurl){
         return false;
     }
-    clist_all.push_back(s3fscurl);
+    clist_all.push_back(std::move(s3fscurl));
 
     return true;
 }
 
 int S3fsMultiCurl::MultiPerform()
 {
-    std::vector<pthread_t>   threads;
-    bool                     success = true;
+    std::map<std::thread::id, std::pair<std::thread, std::future<int>>> threads;
+    int                      result = 0;
     bool                     isMultiHead = false;
     Semaphore                sem(GetMaxParallelism());
-    int                      rc;
 
-    for(s3fscurllist_t::iterator iter = clist_req.begin(); iter != clist_req.end(); ++iter) {
-        pthread_t   thread;
-        S3fsCurl*   s3fscurl = *iter;
+    for(auto iter = clist_req.cbegin(); iter != clist_req.cend(); ++iter) {
+        S3fsCurl*   s3fscurl = iter->get();
         if(!s3fscurl){
             continue;
         }
@@ -145,20 +129,15 @@ int S3fsMultiCurl::MultiPerform()
         sem.wait();
 
         {
-            AutoLock lock(&completed_tids_lock);
-            for(std::vector<pthread_t>::iterator it = completed_tids.begin(); it != completed_tids.end(); ++it){
-                void*   retval;
-    
-                rc = pthread_join(*it, &retval);
-                if (rc) {
-                    success = false;
-                    S3FS_PRN_ERR("failed pthread_join - rc(%d) %s", rc, strerror(rc));
-                } else {
-                    long int_retval = reinterpret_cast<long>(retval);
-                    if (int_retval && !(int_retval == -ENOENT && isMultiHead)) {
-                        S3FS_PRN_WARN("thread terminated with non-zero return code: %ld", int_retval);
-                    }
+            const std::lock_guard<std::mutex> lock(completed_tids_lock);
+            for(const auto &thread_id : completed_tids){
+                auto it = threads.find(thread_id);
+                it->second.first.join();
+                long int int_retval = it->second.second.get();
+                if (int_retval && !(int_retval == -ENOENT && isMultiHead)) {
+                    S3FS_PRN_WARN("thread terminated with non-zero return code: %ld", int_retval);
                 }
+                threads.erase(it);
             }
             completed_tids.clear();
         }
@@ -168,45 +147,39 @@ int S3fsMultiCurl::MultiPerform()
 
         isMultiHead |= s3fscurl->GetOp() == "HEAD";
 
-        rc = pthread_create(&thread, NULL, S3fsMultiCurl::RequestPerformWrapper, static_cast<void*>(s3fscurl));
-        if (rc != 0) {
-            success = false;
-            S3FS_PRN_ERR("failed pthread_create - rc(%d)", rc);
-            break;
-        }
-        threads.push_back(thread);
+        std::promise<int> promise;
+        std::future<int> future = promise.get_future();
+        std::thread thread(S3fsMultiCurl::RequestPerformWrapper, s3fscurl, std::move(promise));
+        auto thread_id = thread.get_id();
+        threads.emplace(std::piecewise_construct, std::forward_as_tuple(thread_id), std::forward_as_tuple(std::move(thread), std::move(future)));
     }
 
     for(int i = 0; i < sem.get_value(); ++i){
         sem.wait();
     }
 
-    AutoLock lock(&completed_tids_lock);
-    for (std::vector<pthread_t>::iterator titer = completed_tids.begin(); titer != completed_tids.end(); ++titer) {
-        void*   retval;
-
-        rc = pthread_join(*titer, &retval);
-        if (rc) {
-            success = false;
-            S3FS_PRN_ERR("failed pthread_join - rc(%d)", rc);
-        } else {
-            long int_retval = reinterpret_cast<long>(retval);
-            if (int_retval && !(int_retval == -ENOENT && isMultiHead)) {
-                S3FS_PRN_WARN("thread terminated with non-zero return code: %ld", int_retval);
-            }
+    const std::lock_guard<std::mutex> lock(completed_tids_lock);
+    for(const auto &thread_id : completed_tids){
+        auto it = threads.find(thread_id);
+        it->second.first.join();
+        auto int_retval = it->second.second.get();
+        if (int_retval && !(int_retval == -ENOENT && isMultiHead)) {
+            S3FS_PRN_WARN("thread terminated with non-zero return code: %d", int_retval);
+            result = int_retval;
         }
+        threads.erase(it);
     }
     completed_tids.clear();
 
-    return success ? 0 : -EIO;
+    return result;
 }
 
 int S3fsMultiCurl::MultiRead()
 {
     int result = 0;
 
-    for(s3fscurllist_t::iterator iter = clist_req.begin(); iter != clist_req.end(); ){
-        S3fsCurl* s3fscurl = *iter;
+    for(auto iter = clist_req.begin(); iter != clist_req.end(); ){
+        std::unique_ptr<S3fsCurl> s3fscurl(std::move(*iter));
 
         bool isRetry = false;
         bool isPostpone = false;
@@ -220,7 +193,9 @@ int S3fsMultiCurl::MultiRead()
                 isPostpone = true;
             }else if(400 > responseCode){
                 // add into stat cache
-                if(SuccessCallback && !SuccessCallback(s3fscurl, pSuccessCallbackParam)){
+                // cppcheck-suppress unmatchedSuppression
+                // cppcheck-suppress knownPointerToBool
+                if(SuccessCallback && !SuccessCallback(s3fscurl.get(), pSuccessCallbackParam)){
                     S3FS_PRN_WARN("error from success callback function(%s).", s3fscurl->url.c_str());
                 }
             }else if(400 == responseCode){
@@ -234,7 +209,9 @@ int S3fsMultiCurl::MultiRead()
                     S3FS_PRN_WARN("failed a request(%ld: %s)", responseCode, s3fscurl->url.c_str());
                 }
 				// Call callback function
-                if(NotFoundCallback && !NotFoundCallback(s3fscurl, pNotFoundCallbackParam)){
+                // cppcheck-suppress unmatchedSuppression
+                // cppcheck-suppress knownPointerToBool
+                if(NotFoundCallback && !NotFoundCallback(s3fscurl.get(), pNotFoundCallbackParam)){
                     S3FS_PRN_WARN("error from not found callback function(%s).", s3fscurl->url.c_str());
                 }
             }else if(500 == responseCode){
@@ -250,7 +227,7 @@ int S3fsMultiCurl::MultiRead()
             }
         }else{
             S3FS_PRN_ERR("failed a request(Unknown response code: %s)", s3fscurl->url.c_str());
-            // Reuse partical file
+            // Reuse particular file
             switch(curlCode){
                 case CURLE_OPERATION_TIMEDOUT:
                     isRetry = true;
@@ -271,35 +248,34 @@ int S3fsMultiCurl::MultiRead()
 
         if(isPostpone){
             clist_req.erase(iter);
-            clist_req.push_back(s3fscurl);    // Re-evaluate at the end
+            clist_req.push_back(std::move(s3fscurl));    // Re-evaluate at the end
             iter = clist_req.begin();
         }else{
-            if(!isRetry || 0 != result){
+            if(!isRetry || (!not_abort && 0 != result)){
                 // If an EIO error has already occurred, it will be terminated
                 // immediately even if retry processing is required. 
                 s3fscurl->DestroyCurlHandle();
-                delete s3fscurl;
             }else{
-                S3fsCurl* retrycurl = NULL;
-
                 // Reset offset
                 if(isNeedResetOffset){
-                    S3fsCurl::ResetOffset(s3fscurl);
+                    S3fsCurl::ResetOffset(s3fscurl.get());
                 }
 
                 // For retry
+                std::unique_ptr<S3fsCurl> retrycurl;
+                const S3fsCurl* retrycurl_ptr = retrycurl.get();  // save this due to std::move below
                 if(RetryCallback){
-                    retrycurl = RetryCallback(s3fscurl);
-                    if(NULL != retrycurl){
-                        clist_all.push_back(retrycurl);
+                    retrycurl = RetryCallback(s3fscurl.get());
+                    if(nullptr != retrycurl){
+                        clist_all.push_back(std::move(retrycurl));
                     }else{
                         // set EIO and wait for other parts.
                         result = -EIO;
                     }
                 }
-                if(s3fscurl != retrycurl){
+                // cppcheck-suppress mismatchingContainers
+                if(s3fscurl.get() != retrycurl_ptr){
                     s3fscurl->DestroyCurlHandle();
-                    delete s3fscurl;
                 }
             }
             iter = clist_req.erase(iter);
@@ -307,12 +283,11 @@ int S3fsMultiCurl::MultiRead()
     }
     clist_req.clear();
 
-    if(0 != result){
+    if(!not_abort && 0 != result){
         // If an EIO error has already occurred, clear all retry objects.
-        for(s3fscurllist_t::iterator iter = clist_all.begin(); iter != clist_all.end(); ++iter){
-            S3fsCurl* s3fscurl = *iter;
+        for(auto iter = clist_all.cbegin(); iter != clist_all.cend(); ++iter){
+            S3fsCurl* s3fscurl = iter->get();
             s3fscurl->DestroyCurlHandle();
-            delete s3fscurl;
         }
         clist_all.clear();
     }
@@ -331,10 +306,8 @@ int S3fsMultiCurl::Request()
     while(!clist_all.empty()){
         // set curl handle to multi handle
         int                      result;
-        s3fscurllist_t::iterator iter;
-        for(iter = clist_all.begin(); iter != clist_all.end(); ++iter){
-            S3fsCurl* s3fscurl = *iter;
-            clist_req.push_back(s3fscurl);
+        for(auto iter = clist_all.begin(); iter != clist_all.end(); ++iter){
+            clist_req.push_back(std::move(*iter));
         }
         clist_all.clear();
 
@@ -359,30 +332,31 @@ int S3fsMultiCurl::Request()
 //
 // thread function for performing an S3fsCurl request
 //
-void* S3fsMultiCurl::RequestPerformWrapper(void* arg)
+void S3fsMultiCurl::RequestPerformWrapper(S3fsCurl* s3fscurl, std::promise<int> promise)
 {
-    S3fsCurl* s3fscurl= static_cast<S3fsCurl*>(arg);
-    void*     result  = NULL;
+    int result = 0;
     if(!s3fscurl){
-        return reinterpret_cast<void*>(static_cast<intptr_t>(-EIO));
+        // this doesn't signal completion but also never happens
+        promise.set_value(-EIO);
+        return;
     }
     if(s3fscurl->fpLazySetup){
         if(!s3fscurl->fpLazySetup(s3fscurl)){
             S3FS_PRN_ERR("Failed to lazy setup, then respond EIO.");
-            result  = reinterpret_cast<void*>(static_cast<intptr_t>(-EIO));
+            result = -EIO;
         }
     }
 
     if(!result){
-        result = reinterpret_cast<void*>(static_cast<intptr_t>(s3fscurl->RequestPerform()));
+        result = s3fscurl->RequestPerform();
         s3fscurl->DestroyCurlHandle(true, false);
     }
 
-    AutoLock  lock(s3fscurl->completed_tids_lock);
-    s3fscurl->completed_tids->push_back(pthread_self());
+    const std::lock_guard<std::mutex> lock(*s3fscurl->completed_tids_lock);
+    s3fscurl->completed_tids->push_back(std::this_thread::get_id());
     s3fscurl->sem->post();
 
-    return result;
+    promise.set_value(result);
 }
 
 /*
