@@ -21,14 +21,15 @@
 #ifndef S3FS_CURL_UTIL_H_
 #define S3FS_CURL_UTIL_H_
 
+#include <cstdint>
 #include <curl/curl.h>
+#include <string>
 
-class sse_type_t;
+enum class sse_type_t : uint8_t;
 
 //----------------------------------------------
 // Functions
 //----------------------------------------------
-struct curl_slist* curl_slist_sort_insert(struct curl_slist* list, const char* data);
 struct curl_slist* curl_slist_sort_insert(struct curl_slist* list, const char* key, const char* value);
 struct curl_slist* curl_slist_remove(struct curl_slist* list, const char* key);
 std::string get_sorted_header_keys(const struct curl_slist* list);
@@ -43,7 +44,7 @@ std::string url_to_host(const std::string &url);
 std::string get_bucket_host();
 const char* getCurlDebugHead(curl_infotype type);
 
-bool etag_equals(std::string s1, std::string s2);
+bool etag_equals(const std::string& s1, const std::string& s2);
 
 #endif // S3FS_CURL_UTIL_H_
 
