@@ -18,14 +18,18 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include <algorithm>
+#include <cerrno>
 #include <cstdio>
 #include <cstdlib>
-#include <errno.h>
+#include <memory>
+#include <string>
 #include <sys/stat.h>
 #include <unistd.h>
 
 #include "common.h"
 #include "s3fs_logger.h"
+#include "s3fs_util.h"
 #include "fdcache_fdinfo.h"
 #include "fdcache_pseudofd.h"
 #include "fdcache_entity.h"
@@ -47,38 +51,36 @@ int PseudoFdInfo::opt_max_threads = -1;
 //
 void* PseudoFdInfo::MultipartUploadThreadWorker(void* arg)
 {
-    pseudofdinfo_thparam*   pthparam = static_cast<pseudofdinfo_thparam*>(arg);
+    std::unique_ptr<pseudofdinfo_thparam> pthparam(static_cast<pseudofdinfo_thparam*>(arg));
     if(!pthparam || !(pthparam->ppseudofdinfo)){
-        delete pthparam;
         return reinterpret_cast<void*>(-EIO);
     }
     S3FS_PRN_INFO3("Upload Part Thread [tpath=%s][start=%lld][size=%lld][part=%d]", pthparam->path.c_str(), static_cast<long long>(pthparam->start), static_cast<long long>(pthparam->size), pthparam->part_num);
 
     int       result;
-    S3fsCurl* s3fscurl;
     {
-        AutoLock auto_lock(&(pthparam->ppseudofdinfo->upload_list_lock));
+        const std::lock_guard<std::mutex> lock(pthparam->ppseudofdinfo->upload_list_lock);
 
         if(0 != (result = pthparam->ppseudofdinfo->last_result)){
             S3FS_PRN_DBG("Already occurred error, thus this thread worker is exiting.");
 
-            if(!pthparam->ppseudofdinfo->CompleteInstruction(result, AutoLock::ALREADY_LOCKED)){    // result will be overwritten with the same value.
+            if(!pthparam->ppseudofdinfo->CompleteInstruction(result)){    // result will be overwritten with the same value.
                 result = -EIO;
             }
-            delete pthparam;
             return reinterpret_cast<void*>(result);
         }
     }
 
     // setup and make curl object
-    if(NULL == (s3fscurl = S3fsCurl::CreateParallelS3fsCurl(pthparam->path.c_str(), pthparam->upload_fd, pthparam->start, pthparam->size, pthparam->part_num, pthparam->is_copy, pthparam->petag, pthparam->upload_id, result))){
+    std::unique_ptr<S3fsCurl> s3fscurl(S3fsCurl::CreateParallelS3fsCurl(pthparam->path.c_str(), pthparam->upload_fd, pthparam->start, pthparam->size, pthparam->part_num, pthparam->is_copy, pthparam->petag, pthparam->upload_id, result));
+    if(nullptr == s3fscurl){
         S3FS_PRN_ERR("failed creating s3fs curl object for uploading [path=%s][start=%lld][size=%lld][part=%d]", pthparam->path.c_str(), static_cast<long long>(pthparam->start), static_cast<long long>(pthparam->size), pthparam->part_num);
 
         // set result for exiting
-        if(!pthparam->ppseudofdinfo->CompleteInstruction(result, AutoLock::NONE)){
+        const std::lock_guard<std::mutex> lock(pthparam->ppseudofdinfo->upload_list_lock);
+        if(!pthparam->ppseudofdinfo->CompleteInstruction(result)){
             result = -EIO;
         }
-        delete pthparam;
         return reinterpret_cast<void*>(result);
     }
 
@@ -93,13 +95,12 @@ void* PseudoFdInfo::MultipartUploadThreadWorker(void* arg)
         S3FS_PRN_ERR("failed uploading with error(%d) [path=%s][start=%lld][size=%lld][part=%d]", result, pthparam->path.c_str(), static_cast<long long>(pthparam->start), static_cast<long long>(pthparam->size), pthparam->part_num);
     }
     s3fscurl->DestroyCurlHandle(true, false);
-    delete s3fscurl;
 
     // set result
-    if(!pthparam->ppseudofdinfo->CompleteInstruction(result, AutoLock::NONE)){
+    const std::lock_guard<std::mutex> lock(pthparam->ppseudofdinfo->upload_list_lock);
+    if(!pthparam->ppseudofdinfo->CompleteInstruction(result)){
         S3FS_PRN_WARN("This thread worker is about to end, so it doesn't return an EIO here and runs to the end.");
     }
-    delete pthparam;
 
     return reinterpret_cast<void*>(result);
 }
@@ -107,20 +108,8 @@ void* PseudoFdInfo::MultipartUploadThreadWorker(void* arg)
 //------------------------------------------------
 // PseudoFdInfo methods
 //------------------------------------------------
-PseudoFdInfo::PseudoFdInfo(int fd, int open_flags) : pseudo_fd(-1), physical_fd(fd), flags(0), upload_fd(-1), uploaded_sem(0), instruct_count(0), completed_count(0), last_result(0)
+PseudoFdInfo::PseudoFdInfo(int fd, int open_flags) : pseudo_fd(-1), physical_fd(fd), flags(0), upload_fd(-1), instruct_count(0), completed_count(0), last_result(0), uploaded_sem(0)
 {
-    pthread_mutexattr_t attr;
-    pthread_mutexattr_init(&attr);
-#if S3FS_PTHREAD_ERRORCHECK
-    pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
-#endif
-    int result;
-    if(0 != (result = pthread_mutex_init(&upload_list_lock, &attr))){
-        S3FS_PRN_CRIT("failed to init upload_list_lock: %d", result);
-        abort();
-    }
-    is_lock_init = true;
-
     if(-1 != physical_fd){
         pseudo_fd = PseudoFdManager::Get();
         flags     = open_flags;
@@ -129,23 +118,24 @@ PseudoFdInfo::PseudoFdInfo(int fd, int open_flags) : pseudo_fd(-1), physical_fd(
 
 PseudoFdInfo::~PseudoFdInfo()
 {
-    Clear();        // call before destrying the mutex
-
-    if(is_lock_init){
-      int result;
-      if(0 != (result = pthread_mutex_destroy(&upload_list_lock))){
-          S3FS_PRN_CRIT("failed to destroy upload_list_lock: %d", result);
-          abort();
-      }
-      is_lock_init = false;
-    }
+    Clear();        // call before destroying the mutex
 }
 
 bool PseudoFdInfo::Clear()
 {
-    if(!CancelAllThreads() || !ResetUploadInfo(AutoLock::NONE)){
+    // cppcheck-suppress unmatchedSuppression
+    // cppcheck-suppress knownConditionTrueFalse
+    if(!CancelAllThreads()){
         return false;
     }
+    {
+        const std::lock_guard<std::mutex> lock(upload_list_lock);
+        // cppcheck-suppress unmatchedSuppression
+        // cppcheck-suppress knownConditionTrueFalse
+        if(!ResetUploadInfo()){
+            return false;
+        }
+    }
     CloseUploadFd();
 
     if(-1 != pseudo_fd){
@@ -157,18 +147,29 @@ bool PseudoFdInfo::Clear()
     return true;
 }
 
+bool PseudoFdInfo::IsUploadingHasLock() const
+{
+    return !upload_id.empty();
+}
+
+bool PseudoFdInfo::IsUploading() const
+{
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
+    return IsUploadingHasLock();
+}
+
 void PseudoFdInfo::CloseUploadFd()
 {
-    AutoLock auto_lock(&upload_list_lock);
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
 
     if(-1 != upload_fd){
         close(upload_fd);
     }
 }
 
-bool PseudoFdInfo::OpenUploadFd(AutoLock::Type type)
+bool PseudoFdInfo::OpenUploadFd()
 {
-    AutoLock auto_lock(&upload_list_lock, type);
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
 
     if(-1 != upload_fd){
         // already initialized
@@ -180,19 +181,25 @@ bool PseudoFdInfo::OpenUploadFd(AutoLock::Type type)
     }
 
     // duplicate fd
-    if(-1 == (upload_fd = dup(physical_fd)) || 0 != lseek(upload_fd, 0, SEEK_SET)){
+    int fd;
+    if(-1 == (fd = dup(physical_fd))){
         S3FS_PRN_ERR("Could not duplicate physical file descriptor(errno=%d)", errno);
-        if(-1 != upload_fd){
-            close(upload_fd);
-        }
+        return false;
+    }
+    scope_guard guard([&]() { close(fd); });
+
+    if(0 != lseek(fd, 0, SEEK_SET)){
+        S3FS_PRN_ERR("Could not seek physical file descriptor(errno=%d)", errno);
         return false;
     }
     struct stat st;
-    if(-1 == fstat(upload_fd, &st)){
+    if(-1 == fstat(fd, &st)){
         S3FS_PRN_ERR("Invalid file descriptor for uploading(errno=%d)", errno);
-        close(upload_fd);
         return false;
     }
+
+    guard.dismiss();
+    upload_fd = fd;
     return true;
 }
 
@@ -232,18 +239,20 @@ bool PseudoFdInfo::Readable() const
 bool PseudoFdInfo::ClearUploadInfo(bool is_cancel_mp)
 {
     if(is_cancel_mp){
+        // cppcheck-suppress unmatchedSuppression
+        // cppcheck-suppress knownConditionTrueFalse
         if(!CancelAllThreads()){
             return false;
         }
     }
-    return ResetUploadInfo(AutoLock::NONE);
+
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
+    return ResetUploadInfo();
 }
 
-bool PseudoFdInfo::ResetUploadInfo(AutoLock::Type type)
+bool PseudoFdInfo::ResetUploadInfo()
 {
-    AutoLock auto_lock(&upload_list_lock, type);
-
-    upload_id.erase();
+    upload_id.clear();
     upload_list.clear();
     instruct_count  = 0;
     completed_count = 0;
@@ -252,32 +261,36 @@ bool PseudoFdInfo::ResetUploadInfo(AutoLock::Type type)
     return true;
 }
 
-bool PseudoFdInfo::RowInitialUploadInfo(const std::string& id, bool is_cancel_mp, AutoLock::Type type)
+bool PseudoFdInfo::RowInitialUploadInfo(const std::string& id, bool is_cancel_mp)
 {
-    if(is_cancel_mp && AutoLock::ALREADY_LOCKED == type){
-        S3FS_PRN_ERR("Internal Error: Could not call this with type=AutoLock::ALREADY_LOCKED and is_cancel_mp=true");
-        return false;
-    }
-
     if(is_cancel_mp){
+        // cppcheck-suppress unmatchedSuppression
+        // cppcheck-suppress knownConditionTrueFalse
         if(!ClearUploadInfo(is_cancel_mp)){
             return false;
         }
     }else{
-        if(!ResetUploadInfo(type)){
+        const std::lock_guard<std::mutex> lock(upload_list_lock);
+        // cppcheck-suppress unmatchedSuppression
+        // cppcheck-suppress knownConditionTrueFalse
+        if(!ResetUploadInfo()){
             return false;
         }
     }
 
-    AutoLock auto_lock(&upload_list_lock, type);
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
     upload_id = id;
     return true;
 }
 
-bool PseudoFdInfo::CompleteInstruction(int result, AutoLock::Type type)
+void PseudoFdInfo::IncreaseInstructionCount()
 {
-    AutoLock auto_lock(&upload_list_lock, type);
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
+    ++instruct_count;
+}
 
+bool PseudoFdInfo::CompleteInstruction(int result)
+{
     if(0 != result){
         last_result = result;
     }
@@ -292,27 +305,36 @@ bool PseudoFdInfo::CompleteInstruction(int result, AutoLock::Type type)
     return true;
 }
 
-bool PseudoFdInfo::GetUploadId(std::string& id) const
+bool PseudoFdInfo::GetUploadInfo(std::string& id, int& fd) const
 {
-    if(!IsUploading()){
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
+
+    if(!IsUploadingHasLock()){
         S3FS_PRN_ERR("Multipart Upload has not started yet.");
         return false;
     }
     id = upload_id;
+    fd = upload_fd;
     return true;
 }
 
+bool PseudoFdInfo::GetUploadId(std::string& id) const
+{
+    int fd = -1;
+    return GetUploadInfo(id, fd);
+}
+
 bool PseudoFdInfo::GetEtaglist(etaglist_t& list) const
 {
-    if(!IsUploading()){
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
+
+    if(!IsUploadingHasLock()){
         S3FS_PRN_ERR("Multipart Upload has not started yet.");
         return false;
     }
 
-    AutoLock auto_lock(&upload_list_lock);
-
     list.clear();
-    for(filepart_list_t::const_iterator iter = upload_list.begin(); iter != upload_list.end(); ++iter){
+    for(auto iter = upload_list.cbegin(); iter != upload_list.cend(); ++iter){
         if(iter->petag){
             list.push_back(*(iter->petag));
         }else{
@@ -332,12 +354,13 @@ bool PseudoFdInfo::GetEtaglist(etaglist_t& list) const
 //
 bool PseudoFdInfo::AppendUploadPart(off_t start, off_t size, bool is_copy, etagpair** ppetag)
 {
-    if(!IsUploading()){
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
+
+    if(!IsUploadingHasLock()){
         S3FS_PRN_ERR("Multipart Upload has not started yet.");
         return false;
     }
 
-    AutoLock auto_lock(&upload_list_lock);
     off_t    next_start_pos = 0;
     if(!upload_list.empty()){
         next_start_pos = upload_list.back().startpos + upload_list.back().size;
@@ -351,9 +374,8 @@ bool PseudoFdInfo::AppendUploadPart(off_t start, off_t size, bool is_copy, etagp
     int partnumber = static_cast<int>(upload_list.size()) + 1;
 
     // add new part
-    etagpair*   petag_entity = etag_entities.add(etagpair(NULL, partnumber));              // [NOTE] Create the etag entity and register it in the list.
-    filepart    newpart(false, physical_fd, start, size, is_copy, petag_entity);
-    upload_list.push_back(newpart);
+    etagpair*   petag_entity = etag_entities.add(etagpair(nullptr, partnumber));              // [NOTE] Create the etag entity and register it in the list.
+    upload_list.emplace_back(false, physical_fd, start, size, is_copy, petag_entity);
 
     // set etag pointer
     if(ppetag){
@@ -368,14 +390,16 @@ bool PseudoFdInfo::AppendUploadPart(off_t start, off_t size, bool is_copy, etagp
 //
 static bool filepart_partnum_compare(const filepart& src1, const filepart& src2)
 {
-    return (src1.get_part_number() <= src2.get_part_number());
+    return src1.get_part_number() < src2.get_part_number();
 }
 
-bool PseudoFdInfo::InsertUploadPart(off_t start, off_t size, int part_num, bool is_copy, etagpair** ppetag, AutoLock::Type type)
+bool PseudoFdInfo::InsertUploadPart(off_t start, off_t size, int part_num, bool is_copy, etagpair** ppetag)
 {
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
+
     //S3FS_PRN_DBG("[start=%lld][size=%lld][part_num=%d][is_copy=%s]", static_cast<long long int>(start), static_cast<long long int>(size), part_num, (is_copy ? "true" : "false"));
 
-    if(!IsUploading()){
+    if(!IsUploadingHasLock()){
         S3FS_PRN_ERR("Multipart Upload has not started yet.");
         return false;
     }
@@ -384,15 +408,12 @@ bool PseudoFdInfo::InsertUploadPart(off_t start, off_t size, int part_num, bool
         return false;
     }
 
-    AutoLock auto_lock(&upload_list_lock, type);
-
     // insert new part
-    etagpair*   petag_entity = etag_entities.add(etagpair(NULL, part_num));
-    filepart    newpart(false, physical_fd, start, size, is_copy, petag_entity);
-    upload_list.push_back(newpart);
+    etagpair*   petag_entity = etag_entities.add(etagpair(nullptr, part_num));
+    upload_list.emplace_back(false, physical_fd, start, size, is_copy, petag_entity);
 
     // sort by part number
-    upload_list.sort(filepart_partnum_compare);
+    std::sort(upload_list.begin(), upload_list.end(), filepart_partnum_compare);
 
     // set etag pointer
     *ppetag = petag_entity;
@@ -404,34 +425,39 @@ bool PseudoFdInfo::InsertUploadPart(off_t start, off_t size, int part_num, bool
 // This method only launches the upload thread.
 // Check the maximum number of threads before calling.
 //
-bool PseudoFdInfo::ParallelMultipartUpload(const char* path, const mp_part_list_t& mplist, bool is_copy, AutoLock::Type type)
+bool PseudoFdInfo::ParallelMultipartUpload(const char* path, const mp_part_list_t& mplist, bool is_copy)
 {
     //S3FS_PRN_DBG("[path=%s][mplist(%zu)]", SAFESTRPTR(path), mplist.size());
 
-    AutoLock auto_lock(&upload_list_lock, type);
-
     if(mplist.empty()){
         // nothing to do
         return true;
     }
-    if(!OpenUploadFd(AutoLock::ALREADY_LOCKED)){
+    if(!OpenUploadFd()){
         return false;
     }
 
-    for(mp_part_list_t::const_iterator iter = mplist.begin(); iter != mplist.end(); ++iter){
+    // Get upload id/fd before loop
+    std::string tmp_upload_id;
+    int         tmp_upload_fd = -1;
+    if(!GetUploadInfo(tmp_upload_id, tmp_upload_fd)){
+        return false;
+    }
+
+    for(auto iter = mplist.cbegin(); iter != mplist.cend(); ++iter){
         // Insert upload part
-        etagpair* petag = NULL;
-        if(!InsertUploadPart(iter->start, iter->size, iter->part_num, is_copy, &petag, AutoLock::ALREADY_LOCKED)){
+        etagpair* petag = nullptr;
+        if(!InsertUploadPart(iter->start, iter->size, iter->part_num, is_copy, &petag)){
             S3FS_PRN_ERR("Failed to insert insert upload part(path=%s, start=%lld, size=%lld, part=%d, copy=%s) to mplist", SAFESTRPTR(path), static_cast<long long int>(iter->start), static_cast<long long int>(iter->size), iter->part_num, (is_copy ? "true" : "false"));
             return false;
         }
 
         // make parameter for my thread
-        pseudofdinfo_thparam* thargs = new pseudofdinfo_thparam;
+        auto* thargs = new pseudofdinfo_thparam;
         thargs->ppseudofdinfo        = this;
         thargs->path                 = SAFESTRPTR(path);
-        thargs->upload_id            = upload_id;
-        thargs->upload_fd            = upload_fd;
+        thargs->upload_id            = tmp_upload_id;
+        thargs->upload_fd            = tmp_upload_fd;
         thargs->start                = iter->start;
         thargs->size                 = iter->size;
         thargs->is_copy              = is_copy;
@@ -439,35 +465,34 @@ bool PseudoFdInfo::ParallelMultipartUpload(const char* path, const mp_part_list_
         thargs->petag                = petag;
 
         // make parameter for thread pool
-        thpoolman_param* ppoolparam  = new thpoolman_param;
-        ppoolparam->args             = thargs;
-        ppoolparam->psem             = &uploaded_sem;
-        ppoolparam->pfunc            = PseudoFdInfo::MultipartUploadThreadWorker;
+        thpoolman_param ppoolparam;
+        ppoolparam.args             = thargs;
+        ppoolparam.psem             = &uploaded_sem;
+        ppoolparam.pfunc            = PseudoFdInfo::MultipartUploadThreadWorker;
 
         // setup instruction
         if(!ThreadPoolMan::Instruct(ppoolparam)){
             S3FS_PRN_ERR("failed setup instruction for uploading.");
-            delete ppoolparam;
             delete thargs;
             return false;
         }
-        ++instruct_count;
+
+        IncreaseInstructionCount();
     }
     return true;
 }
 
-bool PseudoFdInfo::ParallelMultipartUploadAll(const char* path, const mp_part_list_t& upload_list, const mp_part_list_t& copy_list, int& result)
+bool PseudoFdInfo::ParallelMultipartUploadAll(const char* path, const mp_part_list_t& to_upload_list, const mp_part_list_t& copy_list, int& result)
 {
-    S3FS_PRN_DBG("[path=%s][upload_list(%zu)][copy_list(%zu)]", SAFESTRPTR(path), upload_list.size(), copy_list.size());
+    S3FS_PRN_DBG("[path=%s][to_upload_list(%zu)][copy_list(%zu)]", SAFESTRPTR(path), to_upload_list.size(), copy_list.size());
 
     result = 0;
 
-    if(!OpenUploadFd(AutoLock::NONE)){
+    if(!OpenUploadFd()){
         return false;
     }
-
-    if(!ParallelMultipartUpload(path, upload_list, false, AutoLock::NONE) || !ParallelMultipartUpload(path, copy_list, true, AutoLock::NONE)){
-        S3FS_PRN_ERR("Failed setup instruction for uploading(path=%s, upload_list=%zu, copy_list=%zu).", SAFESTRPTR(path), upload_list.size(), copy_list.size());
+    if(!ParallelMultipartUpload(path, to_upload_list, false) || !ParallelMultipartUpload(path, copy_list, true)){
+        S3FS_PRN_ERR("Failed setup instruction for uploading(path=%s, to_upload_list=%zu, copy_list=%zu).", SAFESTRPTR(path), to_upload_list.size(), copy_list.size());
         return false;
     }
 
@@ -499,10 +524,9 @@ ssize_t PseudoFdInfo::UploadBoundaryLastUntreatedArea(const char* path, headers_
     S3FS_PRN_DBG("[path=%s][pseudo_fd=%d][physical_fd=%d]", SAFESTRPTR(path), pseudo_fd, physical_fd);
 
     if(!path || -1 == physical_fd || -1 == pseudo_fd || !pfdent){
-        S3FS_PRN_ERR("pseudo_fd(%d) to physical_fd(%d) for path(%s) is not opened or not writable, or pfdent is NULL.", pseudo_fd, physical_fd, path);
+        S3FS_PRN_ERR("pseudo_fd(%d) to physical_fd(%d) for path(%s) is not opened or not writable, or pfdent is nullptr.", pseudo_fd, physical_fd, path);
         return -EBADF;
     }
-    AutoLock auto_lock(&upload_list_lock);
 
     //
     // Get last update untreated area
@@ -565,11 +589,11 @@ ssize_t PseudoFdInfo::UploadBoundaryLastUntreatedArea(const char* path, headers_
         S3fsCurl    s3fscurl(true);
         std::string tmp_upload_id;
         int         result;
-        if(0 != (result = s3fscurl.PreMultipartPostRequest(path, meta, tmp_upload_id, true))){
+        if(0 != (result = s3fscurl.PreMultipartPostRequest(path, meta, tmp_upload_id))){
             S3FS_PRN_ERR("failed to setup multipart upload(create upload id) by errno(%d)", result);
             return result;
         }
-        if(!RowInitialUploadInfo(tmp_upload_id, false/* not need to cancel */, AutoLock::ALREADY_LOCKED)){
+        if(!RowInitialUploadInfo(tmp_upload_id, false/* not need to cancel */)){
             S3FS_PRN_ERR("failed to setup multipart upload(set upload id to object)");
             return result;
         }
@@ -581,7 +605,7 @@ ssize_t PseudoFdInfo::UploadBoundaryLastUntreatedArea(const char* path, headers_
     // When canceling(overwriting) a part that has already been uploaded, output it.
     //
     if(S3fsLog::IsS3fsLogDbg()){
-        for(filepart_list_t::const_iterator cancel_iter = cancel_uploaded_list.begin(); cancel_iter != cancel_uploaded_list.end(); ++cancel_iter){
+        for(auto cancel_iter = cancel_uploaded_list.cbegin(); cancel_iter != cancel_uploaded_list.cend(); ++cancel_iter){
             S3FS_PRN_DBG("Cancel uploaded: start(%lld), size(%lld), part number(%d)", static_cast<long long int>(cancel_iter->startpos), static_cast<long long int>(cancel_iter->size), (cancel_iter->petag ? cancel_iter->petag->part_num : -1));
         }
     }
@@ -589,7 +613,7 @@ ssize_t PseudoFdInfo::UploadBoundaryLastUntreatedArea(const char* path, headers_
     //
     // Upload Multipart parts
     //
-    if(!ParallelMultipartUpload(path, to_upload_list, false, AutoLock::ALREADY_LOCKED)){
+    if(!ParallelMultipartUpload(path, to_upload_list, false)){
         S3FS_PRN_ERR("Failed to upload multipart parts.");
         return -EIO;
     }
@@ -612,7 +636,7 @@ int PseudoFdInfo::WaitAllThreadsExit()
     int  result;
     bool is_loop = true;
     {
-        AutoLock auto_lock(&upload_list_lock);
+        const std::lock_guard<std::mutex> lock(upload_list_lock);
         if(0 == instruct_count && 0 == completed_count){
             result  = last_result;
             is_loop = false;
@@ -623,7 +647,7 @@ int PseudoFdInfo::WaitAllThreadsExit()
         // need to wait the worker exiting
         uploaded_sem.wait();
         {
-            AutoLock auto_lock(&upload_list_lock);
+            const std::lock_guard<std::mutex> lock(upload_list_lock);
             if(0 < completed_count){
                 --completed_count;
             }
@@ -642,7 +666,7 @@ bool PseudoFdInfo::CancelAllThreads()
 {
     bool need_cancel = false;
     {
-        AutoLock auto_lock(&upload_list_lock);
+        const std::lock_guard<std::mutex> lock(upload_list_lock);
         if(0 < instruct_count && 0 < completed_count){
             S3FS_PRN_INFO("The upload thread is running, so cancel them and wait for the end.");
             need_cancel = true;
@@ -674,14 +698,14 @@ bool PseudoFdInfo::CancelAllThreads()
 // [NOTE]
 // Maximum multipart upload size must be uploading boundary.
 //
-bool PseudoFdInfo::ExtractUploadPartsFromUntreatedArea(off_t& untreated_start, off_t& untreated_size, mp_part_list_t& to_upload_list, filepart_list_t& cancel_upload_list, off_t max_mp_size)
+bool PseudoFdInfo::ExtractUploadPartsFromUntreatedArea(off_t untreated_start, off_t untreated_size, mp_part_list_t& to_upload_list, filepart_list_t& cancel_upload_list, off_t max_mp_size)
 {
     if(untreated_start < 0 || untreated_size <= 0){
-        S3FS_PRN_ERR("Paramters are wrong(untreated_start=%lld, untreated_size=%lld).", static_cast<long long int>(untreated_start), static_cast<long long int>(untreated_size));
+        S3FS_PRN_ERR("Parameters are wrong(untreated_start=%lld, untreated_size=%lld).", static_cast<long long int>(untreated_start), static_cast<long long int>(untreated_size));
         return false;
     }
 
-    // Initiliaze lists
+    // Initialize lists
     to_upload_list.clear();
     cancel_upload_list.clear();
 
@@ -707,28 +731,32 @@ bool PseudoFdInfo::ExtractUploadPartsFromUntreatedArea(off_t& untreated_start, o
     // Also, it is assumed that it must not be a copy area.
     // So if the areas overlap, include uploaded area as an untreated area.
     //
-    for(filepart_list_t::iterator cur_iter = upload_list.begin(); cur_iter != upload_list.end(); /* ++cur_iter */){
-        // Check overlap
-        if((cur_iter->startpos + cur_iter->size - 1) < aligned_start || (aligned_start + aligned_size - 1) < cur_iter->startpos){
-            // Areas do not overlap
-            ++cur_iter;
+    {
+        const std::lock_guard<std::mutex> lock(upload_list_lock);
 
-        }else{
-            // The areas overlap
-            //
-            // Since the start position of the uploaded area is aligned with the boundary,
-            // it is not necessary to check the start position.
-            // If the uploaded area exceeds the untreated area, expand the untreated area.
-            //
-            if((aligned_start + aligned_size - 1) < (cur_iter->startpos + cur_iter->size - 1)){
-                aligned_size += (cur_iter->startpos + cur_iter->size) - (aligned_start + aligned_size);
+        for(auto cur_iter = upload_list.begin(); cur_iter != upload_list.end(); /* ++cur_iter */){
+            // Check overlap
+            if((cur_iter->startpos + cur_iter->size - 1) < aligned_start || (aligned_start + aligned_size - 1) < cur_iter->startpos){
+                // Areas do not overlap
+                ++cur_iter;
+
+            }else{
+                // The areas overlap
+                //
+                // Since the start position of the uploaded area is aligned with the boundary,
+                // it is not necessary to check the start position.
+                // If the uploaded area exceeds the untreated area, expand the untreated area.
+                //
+                if((aligned_start + aligned_size - 1) < (cur_iter->startpos + cur_iter->size - 1)){
+                    aligned_size += (cur_iter->startpos + cur_iter->size) - (aligned_start + aligned_size);
+                }
+
+                //
+                // Add this to cancel list
+                //
+                cancel_upload_list.push_back(*cur_iter);            // Copy and Push to cancel list
+                cur_iter = upload_list.erase(cur_iter);
             }
-
-            //
-            // Add this to cancel list
-            //
-            cancel_upload_list.push_back(*cur_iter);            // Copy and Push to cancel list
-            cur_iter = upload_list.erase(cur_iter);
         }
     }
 
@@ -736,8 +764,8 @@ bool PseudoFdInfo::ExtractUploadPartsFromUntreatedArea(off_t& untreated_start, o
     // Add upload area to the list
     //
     while(max_mp_size <= aligned_size){
-        int part_num = (aligned_start / max_mp_size) + 1;
-        to_upload_list.push_back(mp_part(aligned_start, max_mp_size, part_num));
+        int part_num = static_cast<int>((aligned_start / max_mp_size) + 1);
+        to_upload_list.emplace_back(aligned_start, max_mp_size, part_num);
 
         aligned_start += max_mp_size;
         aligned_size  -= max_mp_size;
@@ -753,7 +781,8 @@ bool PseudoFdInfo::ExtractUploadPartsFromUntreatedArea(off_t& untreated_start, o
 // to_upload_list       : A list of areas to upload in multipart upload.
 // to_copy_list         : A list of areas for copy upload in multipart upload.
 // to_download_list     : A list of areas that must be downloaded before multipart upload.
-// cancel_upload_list : A list of areas that have already been uploaded and will be canceled(overwritten).
+// cancel_upload_list   : A list of areas that have already been uploaded and will be canceled(overwritten).
+// wait_upload_complete : If cancellation areas exist, this flag is set to true when it is necessary to wait until the upload of those cancellation areas is complete.
 // file_size            : The size of the upload file.
 // use_copy             : Specify true if copy multipart upload is available.
 //
@@ -761,23 +790,24 @@ bool PseudoFdInfo::ExtractUploadPartsFromUntreatedArea(off_t& untreated_start, o
 // The untreated_list in fdentity does not change, but upload_list is changed.
 // (If you want to restore it, you can use cancel_upload_list.)
 //
-bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list, mp_part_list_t& to_upload_list, mp_part_list_t& to_copy_list, mp_part_list_t& to_download_list, filepart_list_t& cancel_upload_list, off_t max_mp_size, off_t file_size, bool use_copy)
+bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list, mp_part_list_t& to_upload_list, mp_part_list_t& to_copy_list, mp_part_list_t& to_download_list, filepart_list_t& cancel_upload_list, bool& wait_upload_complete, off_t max_mp_size, off_t file_size, bool use_copy)
 {
-    AutoLock auto_lock(&upload_list_lock);
+    const std::lock_guard<std::mutex> lock(upload_list_lock);
 
-    // Initiliaze lists
+    // Initialize lists
     to_upload_list.clear();
     to_copy_list.clear();
     to_download_list.clear();
     cancel_upload_list.clear();
+    wait_upload_complete = false;
 
     // Duplicate untreated list
     untreated_list_t dup_untreated_list;
     untreated_list.Duplicate(dup_untreated_list);
 
     // Initialize the iterator of each list first
-    untreated_list_t::iterator dup_untreated_iter = dup_untreated_list.begin();
-    filepart_list_t::iterator  uploaded_iter      = upload_list.begin();
+    auto dup_untreated_iter = dup_untreated_list.begin();
+    auto uploaded_iter      = upload_list.begin();
 
     //
     // Loop to extract areas to upload and download
@@ -825,20 +855,20 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
                     // - Add this untreated area to cur_untreated_list
                     // - Delete this from dup_untreated_list
                     //
-                    cur_untreated_list.push_back(untreatedpart(tmp_untreated_start, tmp_untreated_size));
+                    cur_untreated_list.emplace_back(tmp_untreated_start, tmp_untreated_size);
                     dup_untreated_iter = dup_untreated_list.erase(dup_untreated_iter);
                 }else{
                     //
                     // The untreated area exceeds the end of the current area
                     //
 
-                    // Ajust untreated area
+                    // Adjust untreated area
                     tmp_untreated_size  = (cur_start + cur_size) - tmp_untreated_start;
 
-                    // Add ajusted untreated area to cur_untreated_list
-                    cur_untreated_list.push_back(untreatedpart(tmp_untreated_start, tmp_untreated_size));
+                    // Add adjusted untreated area to cur_untreated_list
+                    cur_untreated_list.emplace_back(tmp_untreated_start, tmp_untreated_size);
 
-                    // Remove this ajusted untreated area from the area pointed
+                    // Remove this adjusted untreated area from the area pointed
                     // to by dup_untreated_iter.
                     dup_untreated_iter->size  = (dup_untreated_iter->start + dup_untreated_iter->size) - (cur_start + cur_size);
                     dup_untreated_iter->start = tmp_untreated_start + tmp_untreated_size;
@@ -860,7 +890,7 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
         // It also assumes that each size of uploaded area must be a maximum upload
         // size.
         //
-        filepart_list_t::iterator overlap_uploaded_iter = upload_list.end();
+        auto overlap_uploaded_iter = upload_list.end();
         for(; uploaded_iter != upload_list.end(); ++uploaded_iter){
             if((cur_start < (uploaded_iter->startpos + uploaded_iter->size)) && (uploaded_iter->startpos < (cur_start + cur_size))){
                 if(overlap_uploaded_iter != upload_list.end()){
@@ -884,7 +914,7 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
         //
         // Create upload/download/cancel/copy list for this current area
         //
-        int part_num = (cur_start / max_mp_size) + 1;
+        int part_num = static_cast<int>((cur_start / max_mp_size) + 1);
         if(cur_untreated_list.empty()){
             //
             // No untreated area was detected in this current area
@@ -905,14 +935,14 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
                     // Copy multipart upload available
                     //
                     S3FS_PRN_DBG("To copy: start=%lld, size=%lld", static_cast<long long int>(cur_start), static_cast<long long int>(cur_size));
-                    to_copy_list.push_back(mp_part(cur_start, cur_size, part_num));
+                    to_copy_list.emplace_back(cur_start, cur_size, part_num);
                 }else{
                     //
                     // This current area needs to be downloaded and uploaded
                     //
                     S3FS_PRN_DBG("To download and upload: start=%lld, size=%lld", static_cast<long long int>(cur_start), static_cast<long long int>(cur_size));
-                    to_download_list.push_back(mp_part(cur_start, cur_size));
-                    to_upload_list.push_back(mp_part(cur_start, cur_size, part_num));
+                    to_download_list.emplace_back(cur_start, cur_size);
+                    to_upload_list.emplace_back(cur_start, cur_size, part_num);
                 }
             }
         }else{
@@ -929,11 +959,16 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
                 // So this current area only needs to be uploaded again.
                 //
                 S3FS_PRN_DBG("Cancel upload: start=%lld, size=%lld", static_cast<long long int>(overlap_uploaded_iter->startpos), static_cast<long long int>(overlap_uploaded_iter->size));
+
+                if(!overlap_uploaded_iter->uploaded){
+                    S3FS_PRN_DBG("This cancel upload area is still uploading, so you must wait for it to complete before starting any Stream uploads.");
+                    wait_upload_complete = true;
+                }
                 cancel_upload_list.push_back(*overlap_uploaded_iter);               // add this uploaded area to cancel_upload_list
                 uploaded_iter = upload_list.erase(overlap_uploaded_iter);           // remove it from upload_list
 
                 S3FS_PRN_DBG("To upload: start=%lld, size=%lld", static_cast<long long int>(cur_start), static_cast<long long int>(cur_size));
-                to_upload_list.push_back(mp_part(cur_start, cur_size, part_num));   // add new uploading area to list
+                to_upload_list.emplace_back(cur_start, cur_size, part_num);   // add new uploading area to list
 
             }else{
                 //
@@ -956,7 +991,7 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
                 off_t changed_start = cur_start;
                 off_t changed_size  = cur_size;
                 bool  first_area    = true;
-                for(untreated_list_t::const_iterator tmp_cur_untreated_iter = cur_untreated_list.begin(); tmp_cur_untreated_iter != cur_untreated_list.end(); ++tmp_cur_untreated_iter, first_area = false){
+                for(auto tmp_cur_untreated_iter = cur_untreated_list.cbegin(); tmp_cur_untreated_iter != cur_untreated_list.cend(); ++tmp_cur_untreated_iter, first_area = false){
                     if(tmp_cur_start < tmp_cur_untreated_iter->start){
                         //
                         // Detected a gap at the start of area
@@ -972,14 +1007,14 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
                             // within 5GB and the remaining area after unification is
                             // larger than the minimum multipart upload size.
                             //
-                            mp_part_list_t::reverse_iterator copy_riter = to_copy_list.rbegin();
+                            auto copy_riter = to_copy_list.rbegin();
 
                             if( (copy_riter->start + copy_riter->size) == tmp_cur_start &&
                                 (copy_riter->size + (tmp_cur_untreated_iter->start - tmp_cur_start)) <= FIVE_GB &&
-                                ((tmp_cur_start + tmp_cur_size) - (tmp_cur_untreated_iter->start - tmp_cur_start)) >= MIN_MULTIPART_SIZE )
+                                ((tmp_cur_start + tmp_cur_size) - tmp_cur_untreated_iter->start) >= MIN_MULTIPART_SIZE )
                             {
                                 //
-                                // Unify to this area to previouse copy area.
+                                // Unify to this area to previous copy area.
                                 //
                                 copy_riter->size += tmp_cur_untreated_iter->start - tmp_cur_start;
                                 S3FS_PRN_DBG("Resize to copy: start=%lld, size=%lld", static_cast<long long int>(copy_riter->start), static_cast<long long int>(copy_riter->size));
@@ -994,7 +1029,7 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
                             // If this area is not unified, need to download this area
                             //
                             S3FS_PRN_DBG("To download: start=%lld, size=%lld", static_cast<long long int>(tmp_cur_start), static_cast<long long int>(tmp_cur_untreated_iter->start - tmp_cur_start));
-                            to_download_list.push_back(mp_part(tmp_cur_start, tmp_cur_untreated_iter->start - tmp_cur_start));
+                            to_download_list.emplace_back(tmp_cur_start, tmp_cur_untreated_iter->start - tmp_cur_start);
                         }
                     }
                     //
@@ -1009,14 +1044,14 @@ bool PseudoFdInfo::ExtractUploadPartsFromAllArea(UntreatedParts& untreated_list,
                 //
                 if(0 < tmp_cur_size){
                     S3FS_PRN_DBG("To download: start=%lld, size=%lld", static_cast<long long int>(tmp_cur_start), static_cast<long long int>(tmp_cur_size));
-                    to_download_list.push_back(mp_part(tmp_cur_start, tmp_cur_size));
+                    to_download_list.emplace_back(tmp_cur_start, tmp_cur_size);
                 }
 
                 //
                 // Set upload area(whole of area) to list
                 //
                 S3FS_PRN_DBG("To upload: start=%lld, size=%lld", static_cast<long long int>(changed_start), static_cast<long long int>(changed_size));
-                to_upload_list.push_back(mp_part(changed_start, changed_size, part_num));
+                to_upload_list.emplace_back(changed_start, changed_size, part_num);
             }
         }
     }
