@@ -21,10 +21,9 @@
 #ifndef S3FS_FDCACHE_PAGE_H_
 #define S3FS_FDCACHE_PAGE_H_
 
-#include <list>
+#include <cstdint>
 #include <sys/types.h>
-
-#include "fdcache_stat.h"
+#include <vector>
 
 //------------------------------------------------
 // Symbols
@@ -51,7 +50,7 @@ struct fdpage
     bool   loaded;
     bool   modified;
 
-    fdpage(off_t start = 0, off_t  size = 0, bool is_loaded = false, bool is_modified = false) :
+    explicit fdpage(off_t start = 0, off_t  size = 0, bool is_loaded = false, bool is_modified = false) :
         offset(start), bytes(size), loaded(is_loaded), modified(is_modified) {}
 
     off_t next() const
@@ -63,11 +62,12 @@ struct fdpage
         return (0 < bytes ? offset + bytes - 1 : 0);
     }
 };
-typedef std::list<struct fdpage> fdpage_list_t;
+typedef std::vector<struct fdpage> fdpage_list_t;
 
 //------------------------------------------------
 // Class PageList
 //------------------------------------------------
+class CacheFileStat;
 class FdEntity;
 
 // cppcheck-suppress copyCtorAndEqOperator
@@ -77,13 +77,14 @@ class PageList
 
     private:
         fdpage_list_t pages;
+        bool          is_shrink;    // [NOTE] true if it has been shrunk even once
 
     public:
-        enum page_status{
-            PAGE_NOT_LOAD_MODIFIED = 0,
-            PAGE_LOADED,
-            PAGE_MODIFIED,
-            PAGE_LOAD_MODIFIED
+        enum class page_status : int8_t {
+            NOT_LOAD_MODIFIED = 0,
+            LOADED,
+            MODIFIED,
+            LOAD_MODIFIED
         };
 
     private:
@@ -93,12 +94,16 @@ class PageList
 
         void Clear();
         bool Parse(off_t new_pos);
+        bool Serialize(CacheFileStat& file, ino_t inode);
 
     public:
         static void FreeList(fdpage_list_t& list);
 
-        explicit PageList(off_t size = 0, bool is_loaded = false, bool is_modified = false);
-        explicit PageList(const PageList& other);
+        explicit PageList(off_t size = 0, bool is_loaded = false, bool is_modified = false, bool shrunk = false);
+        PageList(const PageList&) = delete;
+        PageList(PageList&&) = delete;
+        PageList& operator=(const PageList&) = delete;
+        PageList& operator=(PageList&&) = delete;
         ~PageList();
 
         bool Init(off_t size, bool is_loaded, bool is_modified);
@@ -106,7 +111,7 @@ class PageList
         bool Resize(off_t size, bool is_loaded, bool is_modified);
 
         bool IsPageLoaded(off_t start = 0, off_t size = 0) const;                  // size=0 is checking to end of list
-        bool SetPageLoadedStatus(off_t start, off_t size, PageList::page_status pstatus = PAGE_LOADED, bool is_compress = true);
+        bool SetPageLoadedStatus(off_t start, off_t size, PageList::page_status pstatus = page_status::LOADED, bool is_compress = true);
         bool FindUnloadedPage(off_t start, off_t& resstart, off_t& ressize) const;
         off_t GetTotalUnloadedPageSize(off_t start = 0, off_t size = 0, off_t limit_size = 0) const;   // size=0 is checking to end of list
         size_t GetUnloadedPages(fdpage_list_t& unloaded_list, off_t start = 0, off_t size = 0) const;  // size=0 is checking to end of list
@@ -118,7 +123,7 @@ class PageList
         bool ClearAllModified();
 
         bool Compress();
-        bool Serialize(CacheFileStat& file, bool is_output, ino_t inode);
+        bool Deserialize(CacheFileStat& file, ino_t inode);
         void Dump() const;
         bool CompareSparseFile(int fd, size_t file_size, fdpage_list_t& err_area_list, fdpage_list_t& warn_area_list);
 };
