@@ -59,27 +59,6 @@
 #define S3FS_MALLOCTRIM(pad)
 #endif  // S3FS_MALLOC_TRIM
 
-#define S3FS_XMLFREEDOC(doc) \
-        do{ \
-          xmlFreeDoc(doc); \
-          S3FS_MALLOCTRIM(0); \
-        }while(0)
-#define S3FS_XMLFREE(ptr) \
-        do{ \
-          xmlFree(ptr); \
-          S3FS_MALLOCTRIM(0); \
-        }while(0)
-#define S3FS_XMLXPATHFREECONTEXT(ctx) \
-        do{ \
-          xmlXPathFreeContext(ctx); \
-          S3FS_MALLOCTRIM(0); \
-        }while(0)
-#define S3FS_XMLXPATHFREEOBJECT(obj) \
-        do{ \
-          xmlXPathFreeObject(obj); \
-          S3FS_MALLOCTRIM(0); \
-        }while(0)
-
 #endif // S3FS_S3FS_H_
 
 /*
