@@ -22,6 +22,7 @@
 #define S3FS_LOGGER_H_
 
 #include <cstdarg>
+#include <cstdint>
 #include <cstdio>
 #include <string>
 #include <syslog.h>
@@ -42,7 +43,7 @@
 class S3fsLog
 {
     public:
-        enum s3fs_log_level{
+        enum s3fs_log_level : uint8_t {
             LEVEL_CRIT = 0,          // LEVEL_CRIT
             LEVEL_ERR  = 1,          // LEVEL_ERR
             LEVEL_WARN = 3,          // LEVEL_WARNING
@@ -51,15 +52,15 @@ class S3fsLog
         };
 
     protected:
-        static const int         NEST_MAX = 4;
-        static const char* const nest_spaces[NEST_MAX];
-        static const char        LOGFILEENV[];
-        static const char        MSGTIMESTAMP[];
+        static constexpr int         NEST_MAX = 4;
+        static constexpr const char* nest_spaces[NEST_MAX] = {"", "  ", "    ", "      "};
+        static constexpr char        LOGFILEENV[] = "S3FS_LOGFILE";
+        static constexpr char        MSGTIMESTAMP[] = "S3FS_MSGTIMESTAMP";
 
         static S3fsLog*       pSingleton;
         static s3fs_log_level debug_level;
         static FILE*          logfp;
-        static std::string*   plogfile;
+        static std::string    logfile;
         static bool           time_stamp;
 
     protected:
@@ -76,7 +77,7 @@ class S3fsLog
         static bool IsS3fsLogInfo()  { return IsS3fsLogLevel(LEVEL_INFO); }
         static bool IsS3fsLogDbg()   { return IsS3fsLogLevel(LEVEL_DBG);  }
 
-        static int GetSyslogLevel(s3fs_log_level level)
+        static constexpr int GetSyslogLevel(s3fs_log_level level)
         {
             return ( LEVEL_DBG  == (level & LEVEL_DBG) ? LOG_DEBUG   :
                      LEVEL_INFO == (level & LEVEL_DBG) ? LOG_INFO    :
@@ -86,7 +87,7 @@ class S3fsLog
 
         static std::string GetCurrentTime();
 
-        static const char* GetLevelString(s3fs_log_level level)
+        static constexpr const char* GetLevelString(s3fs_log_level level)
         {
             return ( LEVEL_DBG  == (level & LEVEL_DBG) ? "[DBG] " :
                      LEVEL_INFO == (level & LEVEL_DBG) ? "[INF] " :
@@ -94,18 +95,14 @@ class S3fsLog
                      LEVEL_ERR  == (level & LEVEL_DBG) ? "[ERR] " : "[CRT] " );
         }
 
-        static const char* GetS3fsLogNest(int nest)
+        static constexpr const char* GetS3fsLogNest(int nest)
         {
-            if(nest < NEST_MAX){
-                return nest_spaces[nest];
-            }else{
-                return nest_spaces[NEST_MAX - 1];
-            }
+            return nest_spaces[nest < NEST_MAX ? nest : NEST_MAX - 1];
         }
 
         static bool IsSetLogFile()
         {
-            return (NULL != logfp);
+            return (nullptr != logfp);
         }
 
         static FILE* GetOutputLogFile()
@@ -140,6 +137,10 @@ class S3fsLog
 
         explicit S3fsLog();
         ~S3fsLog();
+        S3fsLog(const S3fsLog&) = delete;
+        S3fsLog(S3fsLog&&) = delete;
+        S3fsLog& operator=(const S3fsLog&) = delete;
+        S3fsLog& operator=(S3fsLog&&) = delete;
 };
 
 //-------------------------------------------------------------------
@@ -230,6 +231,34 @@ void s3fs_low_logprn2(S3fsLog::s3fs_log_level level, int nest, const char* file,
 #define S3FS_PRN_CURL(fmt, ...)   S3FS_LOW_CURLDBG(fmt, ##__VA_ARGS__, "")
 #define S3FS_PRN_CACHE(fp, ...)   S3FS_LOW_CACHE(fp, ##__VA_ARGS__, "")
 
+// Macros to print log with fuse context
+#define PRINT_FUSE_CTX(level, indent, fmt, ...) do {                    \
+    if(S3fsLog::IsS3fsLogLevel(level)){                                 \
+        struct fuse_context *ctx = fuse_get_context();                  \
+        if(ctx == NULL){                                                \
+            S3FS_LOW_LOGPRN2(level, indent, fmt, ##__VA_ARGS__);        \
+        }else{                                                          \
+            S3FS_LOW_LOGPRN2(level, indent, fmt"[pid=%u,uid=%u,gid=%u]",\
+                ##__VA_ARGS__,                                          \
+                (unsigned int)(ctx->pid),                               \
+                (unsigned int)(ctx->uid),                               \
+                (unsigned int)(ctx->gid));                              \
+        }                                                               \
+    }                                                                   \
+} while (0)
+
+#define FUSE_CTX_INFO(fmt, ...) do {                            \
+    PRINT_FUSE_CTX(S3fsLog::LEVEL_INFO, 0, fmt, ##__VA_ARGS__); \
+} while (0)
+
+#define FUSE_CTX_INFO1(fmt, ...) do {                           \
+    PRINT_FUSE_CTX(S3fsLog::LEVEL_INFO, 1, fmt, ##__VA_ARGS__); \
+} while (0)
+
+#define FUSE_CTX_DBG(fmt, ...) do {                                 \
+    PRINT_FUSE_CTX(S3fsLog::LEVEL_DBG, 0, fmt, ##__VA_ARGS__);  \
+} while (0)
+
 #endif // S3FS_LOGGER_H_
 
 /*
