@@ -71,6 +71,16 @@ else
 fi
 export SED_BUFFER_FLAG="--unbuffered"
 
+# [NOTE]
+# Specifying cache disable option depending on stat(coreutils) version
+# TODO: investigate why this is necessary #2327
+#
+if stat --cached=never / >/dev/null 2>&1; then
+    STAT_BIN=(stat --cache=never)
+else
+    STAT_BIN=(stat)
+fi
+
 function get_xattr() {
     if [ "$(uname)" = "Darwin" ]; then
         xattr -p "$1" "$2"
@@ -95,11 +105,19 @@ function del_xattr() {
     fi
 }
 
+function get_inode() {
+    if [ "$(uname)" = "Darwin" ]; then
+        "${STAT_BIN[@]}" -f "%i" "$1"
+    else
+        "${STAT_BIN[@]}" --format "%i" "$1"
+    fi
+}
+
 function get_size() {
     if [ "$(uname)" = "Darwin" ]; then
-        stat -f "%z" "$1"
+        "${STAT_BIN[@]}" -f "%z" "$1"
     else
-        stat -c %s "$1"
+        "${STAT_BIN[@]}" --format "%s" "$1"
     fi
 }
 
@@ -107,20 +125,21 @@ function check_file_size() {
     local FILE_NAME="$1"
     local EXPECTED_SIZE="$2"
 
-    # Verify file is zero length via metadata
+    # Verify file length via metadata
     local size
     size=$(get_size "${FILE_NAME}")
     if [ "${size}" -ne "${EXPECTED_SIZE}" ]
     then
-        echo "error: expected ${FILE_NAME} to be zero length"
+        echo "error: expected ${FILE_NAME} to be ${EXPECTED_SIZE} length but was ${size} via metadata"
         return 1
     fi
 
-    # Verify file is zero length via data
-    size=$(wc -c < "${FILE_NAME}")
+    # Verify file length via data
+    # shellcheck disable=SC2002
+    size=$(cat "${FILE_NAME}" | wc -c)
     if [ "${size}" -ne "${EXPECTED_SIZE}" ]
     then
-        echo "error: expected ${FILE_NAME} to be ${EXPECTED_SIZE} length, got ${size}"
+        echo "error: expected ${FILE_NAME} to be ${EXPECTED_SIZE} length, got ${size} via data"
         return 1
     fi
 }
@@ -281,45 +300,53 @@ function run_suite {
 function get_ctime() {
     # ex: "1657504903.019784214"
     if [ "$(uname)" = "Darwin" ]; then
-        stat -f "%Fc" "$1"
+        "${STAT_BIN[@]}" -f "%Fc" "$1"
     else
-        stat -c "%.9Z" "$1"
+        "${STAT_BIN[@]}" --format "%.9Z" "$1"
     fi
 }
 
 function get_mtime() {
     # ex: "1657504903.019784214"
     if [ "$(uname)" = "Darwin" ]; then
-        stat -f "%Fm" "$1"
+        "${STAT_BIN[@]}" -f "%Fm" "$1"
     else
-        stat -c "%.9Y" "$1"
+        "${STAT_BIN[@]}" --format "%.9Y" "$1"
     fi
 }
 
 function get_atime() {
     # ex: "1657504903.019784214"
     if [ "$(uname)" = "Darwin" ]; then
-        stat -f "%Fa" "$1"
+        "${STAT_BIN[@]}" -f "%Fa" "$1"
     else
-        stat -c "%0.9X" "$1"
+        "${STAT_BIN[@]}" --format "%.9X" "$1"
     fi
 }
 
 function get_permissions() {
     if [ "$(uname)" = "Darwin" ]; then
-        stat -f "%p" "$1"
+        "${STAT_BIN[@]}" -f "%p" "$1"
     else
-        stat -c "%a" "$1"
+        "${STAT_BIN[@]}" --format "%a" "$1"
+    fi
+}
+
+function get_user_and_group() {
+    if [ "$(uname)" = "Darwin" ]; then
+        stat -f "%u:%g" "$1"
+    else
+        "${STAT_BIN[@]}" --format "%u:%g" "$1"
     fi
 }
 
 function check_content_type() {
     local INFO_STR
-    INFO_STR=$(aws_cli s3api head-object --bucket "${TEST_BUCKET_1}" --key "$1")
-    if [[ "${INFO_STR}" != *"$2"* ]]
+    INFO_STR=$(aws_cli s3api head-object --bucket "${TEST_BUCKET_1}" --key "$1" | jq -r .ContentType)
+    if [ "${INFO_STR}" != "$2" ]
     then
-        echo "moved file content-type is not as expected expected:$2 got:${INFO_STR}"
-        exit 1
+        echo "Expected Content-Type: $2 but got: ${INFO_STR}"
+        return 1
     fi
 }
 
@@ -334,6 +361,17 @@ function aws_cli() {
     if [ -n "${S3FS_PROFILE}" ]; then
         FLAGS="--profile ${S3FS_PROFILE}"
     fi
+
+    if [ "$1" = "s3" ] && [ "$2" != "ls" ] && [ "$2" != "mb" ]; then
+        if s3fs_args | grep -q use_sse=custom; then
+            FLAGS="${FLAGS} --sse-c AES256 --sse-c-key fileb:///tmp/ssekey.bin"
+        fi
+    elif [ "$1" = "s3api" ] && [ "$2" != "head-bucket" ]; then
+        if s3fs_args | grep -q use_sse=custom; then
+            FLAGS="${FLAGS} --sse-customer-algorithm AES256 --sse-customer-key $(cat /tmp/ssekey) --sse-customer-key-md5 $(cat /tmp/ssekeymd5)"
+        fi
+    fi
+
     # [NOTE]
     # AWS_EC2_METADATA_DISABLED for preventing the metadata service(to 169.254.169.254).
     # shellcheck disable=SC2086,SC2068
@@ -359,12 +397,43 @@ function make_random_string() {
     else
         local END_POS=8
     fi
-
-    "${BASE64_BIN}" --wrap=0 < /dev/urandom | tr -d /+ | head -c "${END_POS}"
+    if [ "$(uname)" = "Darwin" ]; then
+        local BASE64_OPT="--break=0"
+    else
+        local BASE64_OPT="--wrap=0"
+    fi
+    "${BASE64_BIN}" "${BASE64_OPT}" < /dev/urandom 2>/dev/null | tr -d /+ | head -c "${END_POS}"
 
     return 0
 }
 
+function s3fs_args() {
+    if [ "$(uname)" = "Darwin" ]; then
+        ps -o args -p "${S3FS_PID}" | tail -n +2
+    else
+        ps -o args -p "${S3FS_PID}" --no-headers
+    fi
+}
+
+#
+# $1:    sleep seconds
+# $2:    OS type(ex. 'Darwin', unset(means all os type))
+#
+# [NOTE] macos fuse-t
+# macos fuse-t mounts over NFS, and the mtime/ctime/atime attribute
+# values are in seconds(not m/u/n-sec).
+# Therefore, unlike tests on other OSs, we have to wait at least 1
+# second.
+# This function is called primarily for this purpose.
+#
+function wait_ostype() {
+    if [ -z "$2" ] || uname | grep -q "$2"; then
+        if [ -n "$1" ] && ! (echo "$1" | grep -q '[^0-9]'); then
+            sleep "$1"
+        fi
+    fi
+}
+
 #
 # Local variables:
 # tab-width: 4
